# Orca
#
# Copyright 2004-2009 Sun Microsystems Inc.
# Copyright 2010-2013 The Orca Team
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=too-many-lines

"""Labels for Orca's GUIs. These have been put in their own module so that we
can present them in the correct language when users change the language on the
fly without having to reload a bunch of modules."""

from .orca_i18n import C_, _, ngettext  # pylint: disable=import-error

# Translators: This string appears on a button in a dialog. "Activating" the
# selected item will perform the action that one would expect to occur if the
# object were clicked on with the mouse. If the object is a link, activating
# it will bring you to a new page. If the object is a button, activating it
# will press the button. If the object is a combobox, activating it will expand
# it to show all of its contents. And so on.
ACTIVATE = _("_Activate")

# Translators: Some users want to hear additional information when entering
# different types of content. If this checkbox is checked, Orca will announce
# that a blockquote has been entered before speaking the text. At the end of
# the text, Orca will announce that the blockquote is being exited.
ANNOUNCE_BLOCKQUOTES = C_("Context", "Blockquotes")

# Translators: Some users want to hear additional information when entering
# different types of content. If this checkbox is checked, Orca will announce
# that a form has been entered before speaking the contents of that form. At
# the end of the form, Orca will announce that the form is being exited.
ANNOUNCE_FORMS = C_("Context", "Forms")

# Translators: Some users want to hear additional information when entering
# different types of content. If this checkbox is checked, Orca will announce
# when an ARIA landmark has been entered or exited. ARIA landmarks are the W3C
# defined HTML tag attribute 'role' used to identify important part of webpage
# like banners, main context, search, etc.
ANNOUNCE_LANDMARKS = C_("Context", "Landmarks")

# Translators: Some users want to hear additional information when entering
# different types of content. If this checkbox is checked, Orca will announce
# that a list with x items has been entered before speaking the content of that
# list. At the end of the list content, Orca will announce that the list is
# being exited.
ANNOUNCE_LISTS = C_("Context", "Lists")

# Translators: Some users want to hear additional information when entering
# different types of content. If this checkbox is checked, Orca will announce
# that a panel has been entered before speaking the new location. At the end of
# the panel contents, Orca will announce that the panel is being exited. A
# panel is a generic container of objects, such as a group of related form
# fields.
ANNOUNCE_PANELS = C_("Context", "Panels")

# Translators: Some users want to hear additional information when entering
# different types of content. If this checkbox is checked, Orca will announce
# that a table with x rows and y columns has been entered before speaking the
# content of that table. At the end of the table content, Orca will announce
# that the table is being exited.
ANNOUNCE_TABLES = C_("Context", "Tables")

# Translators: This is a shorter version of "Automatic focus mode during caret/structural
# navigation" used on sub-pages where the navigation type context is already clear.
AUTOMATIC_FOCUS_MODE = _("Automatic focus mode")

# Translators: This explains what the "Automatic focus mode" setting does. It appears
# in info boxes on the Caret Navigation and Structural Navigation preferences pages.
# Please translate "Automatic focus mode" consistently with the AUTOMATIC_FOCUS_MODE
# string above.
AUTOMATIC_FOCUS_MODE_INFO = _(
    "Automatic focus mode causes Orca to switch to focus mode when you navigate "
    "into a form field or other interactive widget.",
)

# Translators: This is an informational message on the Native Navigation
# preferences page explaining what native navigation means.
NATIVE_NAVIGATION_INFO = _(
    "Native navigation refers to keyboard commands handled by the application, "
    "such as Tab, Page Up, Page Down, and Enter.",
)

# Translators: This setting controls whether Orca automatically enables sticky
# focus mode for web applications (such as Facebook Messenger) and Electron apps
# (such as Visual Studio Code). When enabled, Orca will detect these applications
# and use focus mode by default. Please translate this string using language
# consistent with `MODE_FOCUS_IS_STICKY = _("Focus mode is sticky.")` in
# messages.py.
AUTO_STICKY_FOCUS_MODE = _("Automatic sticky focus mode for web applications")

# Translators: This explains what the "Automatic sticky focus mode for web
# applications" setting does.
AUTO_STICKY_FOCUS_MODE_INFO = _(
    "Automatic sticky focus mode causes Orca to detect web applications and "
    "Electron apps and stay in focus mode until toggled off by command.",
)

# Translators: A single braille cell on a refreshable braille display consists
# of 8 dots. Dot 7 is the dot in the bottom left corner. If the user selects
# this option, Dot 7 will be used to 'underline' text of interest, e.g. when
# "marking"/indicating that a given word is bold.
BRAILLE_DOT_7 = _("Dot 7")

# Translators: A single braille cell on a refreshable braille display consists
# of 8 dots. Dot 8 is the dot in the bottom right corner. If the user selects
# this option, Dot 8 will be used to 'underline' text of interest,  e.g. when
# "marking"/indicating that a given word is bold.
BRAILLE_DOT_8 = _("Dot 8")

# Translators: A single braille cell on a refreshable braille display consists
# of 8 dots. Dots 7-8 are the dots at the bottom. If the user selects this
# option, Dots 7-8 will be used to 'underline' text of interest,  e.g. when
# "marking"/indicating that a given word is bold.
BRAILLE_DOT_7_8 = _("Dots 7 and 8")

# Translators: This option refers to the dot or dots in braille which will be
# used to underline certain characters.
BRAILLE_DOT_NONE = C_("braille dots", "None")

# Translators: This is the label for a button in a dialog.
BTN_CANCEL = _("_Cancel")

# Translators: This is the label for a button in a dialog.
BTN_JUMP_TO = _("_Jump to")

# Translators: This is the label for a button in a dialog.
BTN_OK = _("_OK")

# Translators: This is the label for a button in a dialog.
BTN_SAVE = _("Save")

# Translators: This is the label for a button in a dialog.
BTN_CLOSE_WITHOUT_SAVING = _("Close without saving")

# Translators: This is the format for a menu item that applies settings to the
# specified profile. The %s will be replaced with the profile name.
MENU_APPLY_PROFILE = _("Apply to %s")

# Translators: This is the format for a menu item that saves settings to the
# specified profile and closes the dialog. The %s will be replaced with the
# profile name.
MENU_SAVE_PROFILE = _("Save to %s and close")

# Translators: This is the label for a menu item to remove a profile.
MENU_REMOVE_PROFILE = _("Remove Profile...")

# Translators: This is the label for a menu item to rename a profile.
MENU_RENAME = _("Rename...")

# Translators: This is the accessible name for the main menu button in the
# preferences dialog. The menu contains actions like Help, Apply, Save, and Cancel.
MENU_BUTTON_OPTIONS = _("Options")

# Translators: This is the primary message in a dialog asking the user to confirm
# closing the preferences window without saving changes.
PREFERENCES_CLOSE_WITHOUT_SAVE = _("Save Changes?")

# Translators: This is the secondary message in a dialog asking the user to confirm
# closing the preferences window without saving changes.
PREFERENCES_CHANGES_WILL_BE_LOST = _("Your changes will be lost if you don't save them.")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text.
# This message appears in a dialog when the user closes the preferences
# after switching to a different profile.
PREFERENCES_PROFILE_SWITCHED = _("You switched profiles.")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text.
# This is a button label in the profile switch dialog. %s is the profile name.
PROFILE_USE = C_("profile", "Use %s")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text.
# This is a button label in the profile switch dialog. %s is the profile name.
PROFILE_SWITCH_BACK_TO = _("Switch back to %s")

# Translators: Orca uses Speech Dispatcher to present content to users via
# text-to-speech. Speech Dispatcher has a feature to control how capital
# letters are presented: Do nothing at all, say the word 'capital' prior to
# presenting a capital letter (which Speech Dispatcher refers to as 'spell'),
# or play a tone (which Speech Dispatcher refers to as a sound 'icon'.) This
# string to be translated appears as a combo box item in Orca's Preferences.
CAPITALIZATION_STYLE_ICON = C_("capitalization style", "Icon")

# Translators: Orca uses Speech Dispatcher to present content to users via
# text-to-speech. Speech Dispatcher has a feature to control how capital
# letters are presented: Do nothing at all, say the word 'capital' prior to
# presenting a capital letter (which Speech Dispatcher refers to as 'spell'),
# or play a tone (which Speech Dispatcher refers to as a sound 'icon'.) This
# string to be translated appears as a combo box item in Orca's Preferences.
CAPITALIZATION_STYLE_NONE = C_("capitalization style", "None")

# Translators: Orca uses Speech Dispatcher to present content to users via
# text-to-speech. Speech Dispatcher has a feature to control how capital
# letters are presented: Do nothing at all, say the word 'capital' prior to
# presenting a capital letter (which Speech Dispatcher refers to as 'spell'),
# or play a tone (which Speech Dispatcher refers to as a sound 'icon'.) This
# string to be translated appears as a combo box item in Orca's Preferences.
CAPITALIZATION_STYLE_SPELL = C_("capitalization style", "Spell")

# Translators: If this widget is enabled, then Orca will tell you when one of
# your buddies is typing a message.
CHAT_ANNOUNCE_BUDDY_TYPING = _("Announce when your _buddies are typing")

# Translators: If this widget is enabled, then Orca will provide the user with
# chat room specific message histories rather than just a single history which
# contains the latest messages from all the chat rooms that they are in.
CHAT_SEPARATE_MESSAGE_HISTORIES = _("Provide chat room specific _message histories")

# Translators: This is the label of a panel holding options for how messages in
# this application's chat rooms should be spoken. The options are: Speak messages
# from all channels (i.e. even if the chat application doesn't have focus); speak
# messages from the active channel regardless of application focus; speak messages
# from all channels but only if the chat application has focus; speak messages from
# the active channel only if the chat application has focus.
CHAT_SPEAK_MESSAGES_FROM = _("Speak messages from")

# Translators: This is the label of a widget in the preferences dialog. If it is
# selected, Orca will speak all new chat messages as they appear irrespective of
# whether or not the chat application currently has focus. This is the default
# behavior.
CHAT_SPEAK_MESSAGES_ALL = _("All channels when using any application")

# Translators: This is the label of a widget in the preferences dialog. If it is
# selected, Orca will only speak new chat messages for the currently active channel
# (the selected tab or focused chat window), regardless of whether or not the chat
# application currently has focus.
CHAT_SPEAK_MESSAGES_ACTIVE_CHANNEL = _("The active channel when using any application")

# Translators: This is the label of a widget in the preferences dialog. If it is
# selected, Orca will speak all new chat messages as they appear if and only if the
# chat application has focus.
CHAT_SPEAK_MESSAGES_ALL_IF_FOCUSED = _("All channels when using the application")

# Translators: This is the label of a widget in the preferences dialog. If it is
# selected, Orca will only speak new chat messages for the currently active channel
# (the selected tab or focused chat window) when the chat application has focus.
CHAT_SPEAK_MESSAGES_ACTIVE = _("The active channel when using the application")

# Translators: If this widget is enabled, then Orca will speak the name of the
# chat room prior to presenting an incoming message.
CHAT_SPEAK_ROOM_NAME = _("_Speak room name")

# Translators: This setting controls when the chat room name is spoken relative
# to the message content. When enabled, the room name is spoken after the message.
# When disabled, the room name is spoken before the message.
CHAT_SPEAK_ROOM_NAME_LAST = _("Speak room name last")

# Translators: When presenting the content of a line on a web page, Orca by
# default presents the full line, including any links or form fields on that
# line, in order to reflect the on-screen layout as seen by sighted users.
# Not all users like this presentation, however, and prefer to have objects
# treated as if they were on individual lines, such as is done by Windows
# screen readers, so that unrelated objects (e.g. links in a navbar) are not
# all jumbled together. As a result, this is now configurable. If layout mode
# is enabled, Orca will present the full line as it appears on the screen; if
# it is disabled, Orca will treat each object as if it were on a separate line,
# both for presentation and navigation.
CONTENT_LAYOUT_MODE = _("Layout mode")

# Translators: This is an informational message on the Caret Navigation preferences
# page explaining what layout mode is. When translating "layout mode", please use
# terminology consistent with that of `CONTENT_LAYOUT_MODE = _("Layout mode")`
# above.

LAYOUT_MODE_INFO = _(
    "If layout mode is enabled, Orca will present the full line as it "
    "appears on the screen. Otherwise, Orca will treat each object as if "
    "it were on a separate line.",
)

# Translators: This is a label which will appear in the list of available speech
# engines as a special item. It refers to the default engine configured within
# the speech subsystem. Apart from this item, the user will have a chance to
# select a particular speech engine by its real name (Festival, IBMTTS, etc.)
DEFAULT_SYNTHESIZER = _("Default Synthesizer")

# Translators: The pronunciation dictionary allows the user to correct words
# which the speech synthesizer mispronounces (e.g. a person's name, a technical
# word) or doesn't pronounce as the user desires (e.g. an acronym) by providing
# an alternative string. For instance "idk" can be sent to the speech server
# as "I don't know" or "I D K" or "eye dee kay" or whatever causes the user's
# speech synthesizer to say what the user finds most helpful.
PRONUNCIATION_DICTIONARY = _("Pronunciation Dictionary")

# Translators: This is an informational message on the Pronunciation Dictionary
# preferences page explaining what the dictionary does.
PRONUNCIATION_DICTIONARY_INFO = _(
    "Customize how words are spoken. Add a word and its replacement below.",
)

# Translators: This is a label for a column header in Orca's pronunciation
# dictionary. The pronunciation dictionary allows the user to correct words
# which the speech synthesizer mispronounces (e.g. a person's name, a technical
# word) or doesn't pronounce as the user desires (e.g. an acronym) by providing
# an alternative string. The "Actual String" here refers to the word to be
# corrected as it would actually appear in text being read. Example: "LOL".
DICTIONARY_ACTUAL_STRING = _("Actual String")

# Translators: This is a label for a column header in Orca's pronunciation
# dictionary. The pronunciation dictionary allows the user to correct words
# which the speech synthesizer mispronounces (e.g. a person's name, a technical
# word) or doesn't pronounce as the user desires (e.g. an acronym) by providing
# an alternative string. The "Replacement String" here refers to how the user
# would like the "Actual String" to be pronounced by the speech synthesizer.
# Example: "L O L" or "Laughing Out Loud" (for Actual String "LOL").
DICTIONARY_REPLACEMENT_STRING = _("Replacement String")

# Translators: The pronunciation dictionary allows the user to correct words
# which the speech synthesizer mispronounces (e.g. a person's name, a technical
# word) or doesn't pronounce as the user desires (e.g. an acronym) by providing
# an alternative string. For instance "idk" can be sent to the speech server
# as "I don't know" or "I D K" or "eye dee kay" or whatever causes the user's
# speech synthesizer to say what the user finds most helpful. This string is
# the label for the widget to add a new entry.
DICTIONARY_NEW_ENTRY = _("New entry")

# Translators: The pronunciation dictionary allows the user to correct words
# which the speech synthesizer mispronounces (e.g. a person's name, a technical
# word) or doesn't pronounce as the user desires (e.g. an acronym) by providing
# an alternative string. For instance "idk" can be sent to the speech server
# as "I don't know" or "I D K" or "eye dee kay" or whatever causes the user's
# speech synthesizer to say what the user finds most helpful. This string is
# the label for the widget to delete the selected entry.
DICTIONARY_DELETE = _("Delete")

# Translators: The pronunciation dictionary allows the user to correct words
# which the speech synthesizer mispronounces (e.g. a person's name, a technical
# word) or doesn't pronounce as the user desires (e.g. an acronym) by providing
# an alternative string. This message is displayed when the pronunciation
# dictionary is empty.
DICTIONARY_EMPTY = _("No pronunciation entries")

# Translators: Title for the dialog to add a new pronunciation entry.
ADD_NEW_PRONUNCIATION = _("Add New Pronunciation")

# Translators: Title for the dialog to edit an existing pronunciation entry.
EDIT_PRONUNCIATION = _("Edit Pronunciation")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This string is the title
# of the dialog box.
FIND_DIALOG_TITLE = _("Find")
KB_GROUP_FIND = FIND_DIALOG_TITLE

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with the text entry where the user types the term to search for.
FIND_SEARCH_FOR = _("_Search for:")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with a group of options related to where the search should begin. The options
# are to begin the search from the current location or from the top of the window.
FIND_START_FROM = _("Start from:")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with the radio button to begin the search from the current location rather
# than from the top of the window.
FIND_START_AT_CURRENT_LOCATION = _("C_urrent location")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with the radio button to begin the search from the top of the window rather
# than the current location.
FIND_START_AT_TOP_OF_WINDOW = _("_Top of window")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with a group of options related to the direction of the search. The options
# are to search backwards and to wrap.
FIND_SEARCH_DIRECTION = _("Search direction:")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with the widget to perform the search in the reverse direction.
FIND_SEARCH_BACKWARDS = _("Search _backwards")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with the widget to wrap around when the top/bottom of the window has been
# reached.
FIND_WRAP_AROUND = _("_Wrap around")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with a group of options related to what constitutes a match. The options are
# to match case and to match the entire word only.
FIND_MATCH_OPTIONS = _("Match options:")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with the widget to make the search case-sensitive.
FIND_MATCH_CASE = _("_Match case")

# Translators: Orca has a "find" feature which allows the user to search the
# active application for on screen text and widgets. This label is associated
# with the widget to only match if the full word consists of the search term.
FIND_MATCH_ENTIRE_WORD = _("Match _entire word only")

# Translators: This is the label for a spinbutton. This option allows the user
# to specify the number of matched characters that must be present before Orca
# speaks the line that contains the results from an application's Find toolbar.
FIND_MINIMUM_MATCH_LENGTH = _("Minimum length of matched text:")

# Translators: This is the label of a panel containing options for what Orca
# presents when the user is in the Find toolbar of an application, e.g. Firefox.
FIND_OPTIONS = _("Find Options")

# Translators: This is the label for a widget. This option controls whether
# the line that contains the match from an application's Find toolbar should
# always be spoken, or only spoken if it is a different line than the line
# which contained the last match.
FIND_ONLY_SPEAK_CHANGED_LINES = _("Onl_y speak changed lines during find")

# Translators: This is the label for a widget. This option controls whether or
# not Orca will automatically speak the line that contains the match while the
# user is performing a search from the Find toolbar of an application, e.g.
# Firefox.
FIND_SPEAK_RESULTS = _("Speak results during _find")

# Translators: Command is a table column header where the cells in the column
# are a sentence that briefly describes what action Orca will take if and when
# the user invokes that keyboard command.
KB_HEADER_FUNCTION = _("Command")

# Translators: Key Binding is a table column header where the cells in the
# column represent keyboard combinations the user can press to invoke Orca
# commands.
KB_HEADER_KEY_BINDING = _("Key Binding")

# Translators: This string is a label for the group of Orca commands which
# can be used in any setting, task, or application. They are not specific
# to, for instance, web browsing.
KB_GROUP_DEFAULT = C_("keybindings", "Default")

# Translators: This string is a label for the group of Orca commands which
# are related to debugging.
KB_GROUP_DEBUGGING_TOOLS = C_("keybindings", "Debugging Tools")

# Translators: This string is a label for the group of Orca commands which
# are related to its "learn mode". Please use the same translation as done
# in cmdnames.py
KB_GROUP_LEARN_MODE = C_("keybindings", "Learn mode")

# Translators: This string is a label for the group of Orca commands which
# are related to presenting and performing the accessible actions associated
# with the current object.
KB_GROUP_ACTIONS = _("Actions")
ACTIONS_LIST = KB_GROUP_ACTIONS

# Translators: This string is a label for the group of Orca commands which
# are related to documents.
KB_GROUP_DOCUMENTS = _("Documents")

# Translators: This string is a label for the group of Orca commands which
# are related to caret navigation, such as moving by character, word, and line.
# These commands are enabled by default for web content and can be optionally
# toggled on in other applications.
KB_GROUP_CARET_NAVIGATION = _("Caret navigation")

# Translators: This is an informational message on the Caret Navigation
# preferences page explaining what caret navigation does.
CARET_NAVIGATION_INFO = _(
    "When enabled, Orca handles the arrow keys, Home, and End to move through "
    "content by character, word, or line.",
)

# Translators: This string is a label for the group of Orca commands which
# are related to chat applications such as instant messaging.
KB_GROUP_CHAT = _("Chat")

# Translators: This string is a label for the group of Orca commands which
# are related to the clipboard.
KB_GROUP_CLIPBOARD = _("Clipboard")

# Translators: A live region is an area of a web page that is periodically
# updated, e.g. stock ticker. https://w3c.github.io/aria/#dfn-live-region
KB_GROUP_LIVE_REGIONS = _("Live regions")

# Translators: Orca has a sleep mode which causes Orca to essentially behave as
# if it were not running for a given application. Some use cases include self-
# voicing apps with associated commands (e.g. ChromeVox) and VMs. In the former
# case, the self-voicing app is expected to provide all needed commands as well
# as speech and braille. In the latter case, we want to ensure that Orca's
# commands and speech/braille do not interfere with that of the VM and any
# screen reader being used in that VM. Thus when an application is being used
# in sleep mode, nearly all Orca commands become unbound/free, and nothing is
# spoken or brailled. But if the user toggles sleep mode off or switches to
# another application window, Orca commands, speech, and braille immediately
# resume working. This string is a label for the group of Orca commands which
# are related to sleep mode.
KB_GROUP_SLEEP_MODE = _("Sleep mode")

# Translators: This string is a label for the group of Orca commands which
# are related to presenting the object under the mouse pointer in speech
# and/or braille. The translation should be consistent with the string
# used in cmdnames.py.
KB_GROUP_MOUSE_REVIEW = _("Mouse review")

# Translators: This string is a label for the group of Orca commands which
# are related to object navigation.
KB_GROUP_OBJECT_NAVIGATION = _("Object navigation")

# Translators: This string is a label for a group of Orca commands which are
# related to presenting information about the system, such as date, time,
# battery status, CPU status, etc.
KB_GROUP_SYSTEM_INFORMATION = _("System information")

# Translators: This string is a label for the group of Orca commands which
# are related to structural navigation, such as moving to the next heading,
# paragraph, form field, etc. in a given direction.
KB_GROUP_STRUCTURAL_NAVIGATION = _("Structural navigation")

# Translators: This is an informational message on the Structural Navigation
# preferences page explaining what structural navigation does.
STRUCTURAL_NAVIGATION_INFO = _(
    "When enabled, alphanumeric keys can be used to quickly jump between "
    "elements. For example, H moves to the next heading, K to the next link. "
    "Add Shift to move backwards.",
)

# Translators: This is a label for a switch that controls whether structural
# navigation wraps around when reaching the top or bottom of a document.
STRUCTURAL_NAVIGATION_WRAP_AROUND = _("Wrap around")

# Translators: This is a label for a spin button that sets the minimum number
# of characters for an object to be considered "large" during structural
# navigation to large objects.
STRUCTURAL_NAVIGATION_LARGE_OBJECT_LENGTH = _("Large object length")

# Translators: This is an informational message on the Structural Navigation
# preferences page explaining what the "Large object length" setting does.
# When translating "large object", please use terminology consistent with that of
# `STRUCTURAL_NAVIGATION_LARGE_OBJECT_LENGTH = _("Large object length")` above.
# See also cmdnames.py's `LARGE_OBJECT_NEXT = _("Go to next large object")`
# and `LARGE_OBJECT_PREVIOUS = _("Go to previous large object")`.
LARGE_OBJECT_INFO = _(
    "Large object length defines the minimum number of characters an element "
    "must have to be included when using the large object navigation shortcuts.",
)

# Translators: This string is a label for the group of Orca commands which
# are related to table navigation, such as moving to the next cell in a
# given direction.
KB_GROUP_TABLE_NAVIGATION = _("Table navigation")

# Translators: This string is a label for the group of Orca commands which
# are related to presenting information about the current location, such as
# the title, status bar, and default button of the current window; the
# name, role, and location of the currently-focused object; the selected
# text in the currently-focused object; etc.
KB_GROUP_WHERE_AM_I = _("Object details")

# Translators: This string is a label for the group of Orca commands which
# are related to Orca's "flat review" feature. This feature allows the blind
# user to explore the text in a window in a 2D fashion. That is, Orca treats
# all the text from all objects in a window (e.g., buttons, labels, etc.) as
# a sequence of words in a sequence of lines.  The flat review feature allows
# the user to explore this text by the {previous,next} {line,word,character}.
# Those commands are all listed under this group label.
KB_GROUP_FLAT_REVIEW = _("Flat review")

# Translators: This string is a label for the group of Orca commands which
# are related to Orca's speech and verbosity settings. This group of commands
# allows on-the-fly configuration of how much (or little) Orca says about a
# particular object, as well certain aspects of the voice with which things
# are spoken.
KB_GROUP_SPEECH_VERBOSITY = _("Speech and verbosity")

# Translators: the 'flat review' feature of Orca allows the blind user to
# explore the text in a window in a 2D fashion.  That is, Orca treats all
# the text from all objects in a window (e.g., buttons, labels, etc.) as a
# sequence of words in a sequence of lines.  The flat review feature allows
# the user to explore this text by the {previous,next} {line,word,character}.
# Normally the contents are navigated without leaving the application being
# reviewed. There is a command which will place the entire contents of the
# flat review representation into a text view to make it easy to review
# and copy the text. This string is the title of the window with the text view.
FLAT_REVIEW_CONTENTS = _("Flat review contents")

# Translators: This is a label for the setting which determines if Orca will
# use the "desktop" or "laptop" keyboard layout. The desktop layout is
# intended for use with a full-size keyboard, while the laptop layout is
# intended for use with a laptop-style keyboard.
KEYBOARD_LAYOUT = _("Keyboard Layout")

# Translators: This label refers to the keyboard layout (desktop or laptop).
# The desktop layout is intended for use with a full-size keyboard, while the
# laptop layout is intended for use with a laptop-style keyboard.
KEYBOARD_LAYOUT_DESKTOP = _("Desktop")

# Translators: This label refers to the keyboard layout (desktop or laptop).
# The desktop layout is intended for use with a full-size keyboard, while the
# laptop layout is intended for use with a laptop-style keyboard.
KEYBOARD_LAYOUT_LAPTOP = _("Laptop")

# Translators: Orca has a feature to list all of the notification messages
# received, similar to the functionality gnome-shell provides when you press
# Super+M, but it works in all desktop environments. Orca's list is a table
# with two columns, one column for the text of the notification and one
# column for the time of the notification. This string is a column header
# for the text of the notifications.
NOTIFICATIONS_COLUMN_HEADER = C_("notification presenter", "Notifications")

# Translators: Orca has a feature to list all of the notification messages
# received, similar to the functionality gnome-shell provides when you press
# Super+M, but it works in all desktop environments. Orca's list is a table
# with two columns, one column for the text of the notification and one
# column for the time of the notification. This string is a column header
# for the time, which will be relative (e.g. "10 minutes ago") or absolute.
NOTIFICATIONS_RECEIVED_TIME = C_("notification presenter", "Received")

# Translators: This string is a label for the group of Orca commands which
# are associated with presenting notifications.
KB_GROUP_NOTIFICATIONS = _("Notification presenter")

# Translators: Orca's preferences can be configured on a per-application basis,
# allowing users to customize Orca's behavior, keybindings, etc. to work one
# way in LibreOffice and another way in a chat application. This string is the
# title of Orca's application-specific preferences dialog for an application.
# The string substituted in is the accessible name of the application (e.g.
# "Gedit", "Firefox", etc.
PREFERENCES_APPLICATION_TITLE = _("Screen Reader Preferences for %s")

# Translators: This is the accessible name for the list of settings categories
# on the left side of the preferences dialog (e.g. "General", "Speech", "Braille").
PREFERENCES_CATEGORIES = _("Categories")

# Translators: This is a table column header. This column consists of a single
# widget. If the widget is enabled, Orca will indicate the associated item
# or attribute by "marking" it in braille. "Marking" is not the same as writing
# out the word; instead marking refers to adding some other indicator, e.g.
# "underlining" with braille dots 7-8 a word that is bold.
PRESENTATION_MARK_IN_BRAILLE = _("Mark in braille")

# Translators: This is a table column header. The "Speak" column consists of a
# single widget. If the widget is enabled, Orca will speak the associated
# item or attribute (e.g. saying "Bold" as part of the information presented
# when the user gives the Orca command to obtain the format and font details of
# the current text).
PRESENTATION_SPEAK = _("Speak")

# Translators: This is an option in a combo box for how a text attribute should
# be presented. If selected, the attribute will be both spoken and marked in braille.
PRESENTATION_SPEAK_AND_MARK = _("Speak and mark")

# Translators: This is an option in a combo box for how a text attribute should
# be presented. If selected, the attribute will not be presented.
TEXT_ATTRIBUTES_PRESENTATION_NONE = C_("text attributes", "None")

# Translators: This is the message in a dialog informing the user that he/she
# attempted to save a new user profile under a name which already exists.
# A "user profile" is a collection of settings which apply to a given task, such
# as a "Spanish" profile which would use Spanish text-to-speech and Spanish
# braille and selected when reading Spanish content.
PROFILE_CONFLICT_MESSAGE = _(
    "Profile %s already exists.\nContinue updating the existing profile with these new changes?",
)

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text.
# The following string is the name of the default profile which is created
# when Orca is installed.
PROFILE_DEFAULT = C_("Profile", "Default")

# Translators: This is an error message displayed when the user tries to remove
# the default profile.
PROFILE_CANNOT_REMOVE_DEFAULT = _("The default profile cannot be removed.")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text. The
# following string is the title of a dialog in which users can save a newly-
# defined profile.
PROFILE_SAVE_AS_TITLE = _("Save Profile As")

# Translators: This is the label for a button that allows users to create a new
# profile. A "profile" is a named collection of Orca settings that users can
# switch between. Creating a new profile copies the current settings.
PROFILE_CREATE_NEW = _("New Profile")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text. The
# following string is the label for a text entry in which the user enters the
# name of a new settings profile being saved via the 'Save Profile As' dialog.
PROFILE_NAME_LABEL = _("_Profile Name:")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text.
# The following is a label in a dialog informing the user that he/she
# is about to remove a user profile, and action that cannot be undone.
PROFILE_REMOVE_LABEL = _("Remove user profile")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text.
# The following is a message in a dialog informing the user that he/she
# is about to remove a user profile, an action that cannot be undone.
PROFILE_REMOVE_MESSAGE = _(
    "You are about to remove profile %s. "
    "All unsaved settings and settings saved in this "
    "profile will be lost. Do you want to continue "
    "and remove this profile and all related settings?",
)

# Translators: This is a message in a dialog shown when the user tries to create
# a new profile while there are unsaved changes in the preferences dialog.
# Creating a new profile copies the saved settings, not the unsaved changes.
PROFILE_CREATE_UNSAVED_WARNING = _(
    "You have unsaved changes. Creating a new profile will copy your "
    "current saved settings, not your unsaved changes. Your unsaved "
    "changes will be lost.\n\nDo you want to continue?",
)

# Translators: Orca has a setting which determines which progress bar updates
# should be announced. Choosing "All" means that Orca will present progress bar
# updates regardless of what application and window they happen to be in.
PROGRESS_BAR_ALL = C_("ProgressBar", "All")

# Translators: Orca has a setting which determines which progress bar updates
# should be announced. Choosing "Application" means that Orca will present
# progress bar updates as long as the progress bar is in the active application
# (but not necessarily in the current window).
PROGRESS_BAR_APPLICATION = C_("ProgressBar", "Application")

# Translators: Orca has a setting which determines which progress bar updates
# should be announced. Choosing "Window" means that Orca will present progress
# bar updates as long as the progress bar is in the active window.
PROGRESS_BAR_WINDOW = C_("ProgressBar", "Window")

# Translators: Orca has a setting which determines how much punctuation should
# be spoken as a user reads a document. The choices are None, Some, Most, and All.
PUNCTUATION_STYLE = _("Punctuation Level")

# Translators: If this setting is chosen, no punctuation symbols will be spoken
# as a user reads a document.
PUNCTUATION_STYLE_NONE = C_("punctuation level", "None")

# Translators: If this setting is chosen, common punctuation symbols (like
# comma, period, question mark) will not be spoken as a user reads a document,
# but less common symbols (such as #, @, $) will.
PUNCTUATION_STYLE_SOME = C_("punctuation level", "Some")

# Translators: If this setting is chosen, the majority of punctuation symbols
# will be spoken as a user reads a document.
PUNCTUATION_STYLE_MOST = C_("punctuation level", "Most")

# Translators: If this setting is chosen, all punctuation symbols will be spoken
# as a user reads a document.
PUNCTUATION_STYLE_ALL = C_("punctuation level", "All")

# Translators: If this setting is chosen and the user is reading over an entire
# document, Orca will pause at the end of each line.
SAY_ALL_STYLE_LINE = _("Line")

# Translators: If this setting is chosen and the user is reading over an entire
# document, Orca will pause at the end of each sentence.
SAY_ALL_STYLE_SENTENCE = _("Sentence")

# Translators: This is the label for a group of settings in the Say All
# preferences page. It describes different methods for rewinding and fast
# forwarding during Say All (e.g., using arrow keys or structural navigation).
SAY_ALL_REWIND_AND_FAST_FORWARD_BY = _("Rewind and Fast Forward By")

# Translators: Orca has a Say All feature which speaks the entire document.
# Normally, pressing any key will interrupt Say All presentation. However, if
# this option is enabled, Up Arrow and Down Arrow can be used to quickly move
# within the document to re-hear something which was just read or skip past
# something of no interest.
SAY_ALL_UP_AND_DOWN_ARROW = _("Up and down arrow")

# Translators: This is a label for a setting on the Say All preferences page. Orca
# provides a Say All command that reads content from the current location to the
# end. If this setting is enabled, users can use Orca's structural navigation
# commands (like H/Shift+H to jump to headings, P/Shift+P to jump to paragraphs)
# to jump forward or backward during Say All.
SAY_ALL_STRUCTURAL_NAVIGATION = _("Structural navigation")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a blockquote.
SN_HEADER_BLOCKQUOTE = C_("structural navigation", "Blockquote")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a button.
SN_HEADER_BUTTON = C_("structural navigation", "Button")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the caption of a table.
SN_HEADER_CAPTION = C_("structural navigation", "Caption")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the label of a check box.
SN_HEADER_CHECK_BOX = C_("structural navigation", "Check Box")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text displayed for a web element with an "onClick" handler.
SN_HEADER_CLICKABLE = C_("structural navigation", "Clickable")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the selected item in a combo box.
SN_HEADER_COMBO_BOX = C_("structural navigation", "Combo Box")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the description of an element.
SN_HEADER_DESCRIPTION = C_("structural navigation", "Description")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a heading.
SN_HEADER_HEADING = C_("structural navigation", "Heading")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the title associated with an iframe.
SN_HEADER_IFRAME = C_("structural navigation", "Internal Frame")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text (alt text, title, etc.) associated with an image.
SN_HEADER_IMAGE = C_("structural navigation", "Image")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the label of a form field.
SN_HEADER_LABEL = C_("structural navigation", "Label")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a landmark. ARIA role landmarks are the W3C defined HTML
# tag attribute 'role' used to identify important part of webpage like banners,
# main context, search etc.
SN_HEADER_LANDMARK = C_("structural navigation", "Landmark")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of a column which
# contains the level of a heading. Level will be a "1" for <h1>, a "2" for <h2>,
# and so on.
SN_HEADER_LEVEL = C_("structural navigation", "Level")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a link.
SN_HEADER_LINK = C_("structural navigation", "Link")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a list.
SN_HEADER_LIST = C_("structural navigation", "List")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a list item.
SN_HEADER_LIST_ITEM = C_("structural navigation", "List Item")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of an object.
SN_HEADER_OBJECT = C_("structural navigation", "Object")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the text of a paragraph.
SN_HEADER_PARAGRAPH = C_("structural navigation", "Paragraph")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the label of a radio button.
SN_HEADER_RADIO_BUTTON = C_("structural navigation", "Radio Button")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the role of a widget. Examples include "heading", "paragraph",
# "table", "combo box", etc.
SN_HEADER_ROLE = C_("structural navigation", "Role")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the state of a widget. Examples include "checked"/"not checked",
# "selected"/"not selected", "visited/not visited", etc.
SN_HEADER_STATE = C_("structural navigation", "State")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the URI of a link.
SN_HEADER_URI = C_("structural navigation", "URI")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title for a column which
# contains the value of a form field.
SN_HEADER_VALUE = C_("structural navigation", "Value")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_BLOCKQUOTE = C_("structural navigation", "Blockquotes")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_BUTTON = C_("structural navigation", "Buttons")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_CHECK_BOX = C_("structural navigation", "Check Boxes")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
# "Clickables" are web elements which have an "onClick" handler.
SN_TITLE_CLICKABLE = C_("structural navigation", "Clickables")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_COMBO_BOX = C_("structural navigation", "Combo Boxes")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_ENTRY = C_("structural navigation", "Entries")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_FORM_FIELD = C_("structural navigation", "Form Fields")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_HEADING = C_("structural navigation", "Headings")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_IFRAME = C_("structural navigation", "Internal Frames")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_IMAGE = C_("structural navigation", "Images")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
# Level will be a "1" for <h1>, a "2" for <h2>, and so on.
SN_TITLE_HEADING_AT_LEVEL = C_("structural navigation", "Headings at Level %d")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
# ARIA role landmarks are the W3C defined HTML tag attribute 'role' used to
# identify important part of webpage like banners, main context, search etc.
SN_TITLE_LANDMARK = C_("structural navigation", "Landmarks")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
# A 'large object' is a logical chunk of text, such as a paragraph, a list,
# a table, etc.
SN_TITLE_LARGE_OBJECT = C_("structural navigation", "Large Objects")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_LINK = C_("structural navigation", "Links")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_LIST = C_("structural navigation", "Lists")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_LIST_ITEM = C_("structural navigation", "List Items")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_PARAGRAPH = C_("structural navigation", "Paragraphs")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_RADIO_BUTTON = C_("structural navigation", "Radio Buttons")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_TABLE = C_("structural navigation", "Tables")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_UNVISITED_LINK = C_("structural navigation", "Unvisited Links")

# Translators: Orca has a command that presents a list of structural navigation
# objects in a dialog box so that users can navigate more quickly than they
# could with native keyboard navigation. This is the title of such a dialog box.
SN_TITLE_VISITED_LINK = C_("structural navigation", "Visited Links")

# Translators: When the user loads a new web page, they can optionally have Orca
# automatically start reading the page from beginning to end. This is the label
# of a widget in which users can indicate their preference.
READ_PAGE_UPON_LOAD = _("Automatically start speaking a page when it is first _loaded")

# Translators: When the user loads a new web page, they can optionally have Orca
# automatically summarize details about the page, such as the number of elements
# (landmarks, forms, links, tables, etc.).
PAGE_SUMMARY_UPON_LOAD = _("_Present summary of a page when it is first loaded")

# Translators: Different speech systems and speech engines work differently when
# it comes to handling pauses (e.g. sentence boundaries). This property allows
# the user to specify whether speech should be sent to the speech synthesis
# system immediately when a pause directive is encountered or if it should be
# queued up and sent to the speech synthesis system once the entire set of
# utterances has been calculated.
SPEECH_BREAK_INTO_CHUNKS = _("Insert pauses to break up speech")

# Translators: This string will appear in the list of available voices for the
# current speech engine. "%s" will be replaced by the name of the current speech
# engine, such as "Festival default voice" or "IBMTTS default voice". It refers
# to the default voice configured for given speech engine within the speech
# subsystem. Apart from this item, the list will contain the names of all
# available "real" voices provided by the speech engine.
SPEECH_DEFAULT_VOICE = _("%s default voice")

# Translators: This refers to the voice used by Orca when presenting the content
# of the screen and other messages.
SPEECH_VOICE_TYPE_DEFAULT = C_("VoiceType", "Default")

# Translators: This refers to the voice used by Orca when presenting one or more
# characters which is part of a hyperlink.
SPEECH_VOICE_TYPE_HYPERLINK = C_("VoiceType", "Hyperlink")

# Translators: This refers to the voice used by Orca when presenting information
# which is not displayed on the screen as text, but is still being communicated
# by the system in some visual fashion. For instance, Orca says "misspelled" to
# indicate the presence of the red squiggly line found under a spelling error;
# Orca might say "3 of 6" when a user Tabs into a list of six items and the
# third item is selected. And so on.
SPEECH_VOICE_TYPE_SYSTEM = C_("VoiceType", "System")

# Translators: This refers to the voice used by Orca when presenting one or more
# characters which is written in uppercase.
SPEECH_VOICE_TYPE_UPPERCASE = C_("VoiceType", "Uppercase")

# Translators this label refers to the name of particular speech synthesis
# system. (http://devel.freebsoft.org/speechd)
SPEECH_DISPATCHER = _("Speech Dispatcher")

# Translators this label refers to the name of particular speech synthesis
# system. (https://github.com/eeejay/spiel)
SPIEL = _("Spiel")

# Translators: This is a label for a group of options related to Orca's behavior
# when presenting an application's spell check dialog.
SPELL_CHECK = C_("OptionGroup", "Spell Check")

# Translators: This is a description that appears at the top of the Spell Check
# preferences page. It explains that these settings only work when the screen
# reader can identify the components of an application's spell checker (e.g., the
# misspelled word, suggestions list, etc.).
SPELL_CHECK_DESCRIPTION = _(
    "These settings apply when the screen reader can identify the "
    "elements of the application's spell check dialog.",
)

# Translators: This is a label for a widget associated with an Orca setting.
# When this option is enabled, Orca will spell out the current error in addition
# to speaking it. For example, if the misspelled word is "foo," enabling this
# setting would cause Orca to speak "f o o" after speaking "foo".
SPELL_CHECK_SPELL_ERROR = _("Spell _error")

# Translators: This is a label for a widget associated with an Orca setting.
# When this option is enabled, Orca will spell out the current suggestion in
# addition to speaking it. For example, if the misspelled word is "foo," and
# the first suggestion is "for" enabling this setting would cause Orca to speak
# "f o r" after speaking "for".
SPELL_CHECK_SPELL_SUGGESTION = _("Spell _suggestion")

# Translators: This is a label for a widget associated with an Orca setting.
# When this option is enabled, Orca will present the context (surrounding text,
# typically the sentence or line) in which the mistake occurred.
SPELL_CHECK_PRESENT_CONTEXT = _("Present _context of error")

# Translators: This is a label for an option to tell Orca whether or not it
# should speak the coordinates of the current spreadsheet cell. Coordinates are
# the row and column position within the spreadsheet (i.e. A1, B1, C2 ...)
SPREADSHEET_SPEAK_CELL_COORDINATES = _("Speak spreadsheet cell coordinates")

# Translators: This is a label for an option which controls what Orca speaks when
# presenting selection changes in a spreadsheet. By default, Orca will speak just
# what changed. For instance, if cells A1 through A8 are already selected, and the
# user adds A9 to the selection, Orca by default would just say "A9 selected."
# Some users, however, prefer to have Orca always announce the entire selected range,
# i.e. in the same scenario say "A1 through A9 selected." Those users should enable
# this option.
SPREADSHEET_SPEAK_SELECTED_RANGE = _("Always speak selected spreadsheet range")

# Translators: This is a label for an option for whether or not to speak the
# header of a table cell in document content.
TABLE_SPEAK_CELL_HEADER = _("Speak cell header")

# Translators: This is a label for a group of settings in the Tables tab of the
# Orca Preferences dialog. These settings control whether Orca speaks the entire
# row when the user moves from row to row in different types of tables.
TABLE_ROW_NAVIGATION = _("Row Navigation")

# Translators: This is a label for a group of settings in the Tables tab of the
# Orca Preferences dialog. These settings control what information about table
# cells Orca announces, such as cell coordinates, headers, and spans.
TABLE_CELL_NAVIGATION = _("Cell Navigation")

# Translators: This is a label for an option to tell Orca to skip over empty/
# blank cells when navigating tables in document content.
TABLE_SKIP_BLANK_CELLS = _("Skip _blank cells")

# Translators: This is a label for an option to tell Orca whether or not it
# should speak table cell coordinates in document content.
TABLE_SPEAK_CELL_COORDINATES = _("Speak _cell coordinates")

# Translators: This is a label for an option to tell Orca whether or not it
# should speak the span size of a table cell (e.g., how many rows and columns
# a particular table cell spans in a table).
TABLE_SPEAK_CELL_SPANS = _("Speak _multiple cell spans")

# Translators: This string is associated with a combo box which allows the user
# to select the set of symbols to be used when Orca presents print strings on a
# refreshable braille display. Braille symbols vary from language to language
# due in part to what print letters exist for that language. The other reason
# braille symbols vary is due to which braille contractions get used.
# Contractions are shorter forms of commonly-used letter combinations and
# words. For instance in English there is a single braille symbol for ing (dots
# 3-4-6), and the letter e (dots 1-5) all by itself represents the word every.
# The list of rules which dictate what contractions should be used and whether
# or not they can be used in a particular context are stored in tables provided
# by liblouis.
BRAILLE_CONTRACTION_TABLE = _("Contraction _Table:")

# Translators: Braille flash messages are similar in nature to notifications or
# announcements. They are most commonly used for Orca to communicate
# Orca-specific information to the user via braille, such as confirming the
# toggling of an Orca setting via command. The reason they are called flash
# messages by screen readers is that they are shown on the refreshable braille
# display for only a brief time, after which the original contents of the
# display are restored. This label is for the spin button through which a user
# can customize how long (in seconds) these temporary messages should be
# displayed.
BRAILLE_DURATION_SECS = _("D_uration (secs):")

# Translators: This is the label for a setting which controls whether Orca
# displays the braille indicator symbol at the end of each line of text.
BRAILLE_ENABLE_END_OF_LINE_SYMBOL = _("_End of line symbol")

# Translators: This is a label for a group of settings related to a refreshable
# braille display.
BRAILLE_DISPLAY_SETTINGS = _("Display Settings")

# Translators: This is the label for a setting in the preferences dialog which
# turns braille support on or off.
BRAILLE_ENABLE_BRAILLE_SUPPORT = _("Braille support")

# Translators: If this option is enabled, Orca will adjust the text shown on
# the braille display so that only full words are shown. If it is not enabled,
# Orca uses all of the cells on the display, but some words might not be fully
# shown requiring the user to scroll to see the remainder.
BRAILLE_ENABLE_WORD_WRAP = _("Word wrap")

# Translators: Braille flash messages are similar in nature to notifications or
# announcements in that they are temporarily shown on the refreshable braille
# display. Upon removal of the message, the original contents of the braille
# display are restored. This widget allows the user to toggle this feature.
BRAILLE_ENABLE_FLASH_MESSAGES = _("Flash messages")

# Translators: Braille flash messages are similar in nature to notifications or
# announcements. They are most commonly used for Orca to communicate
# Orca-specific information to the user via braille, such as confirming the
# toggling of an Orca setting via command. The reason they are called flash
# messages by screen readers is that they are shown on the refreshable braille
# display for only a brief time, after which the original contents of the
# display are restored.
BRAILLE_FLASH_MESSAGE_SETTINGS = _("Flash Message Settings")

# Translators: This label is for a frame containing braille indicator settings.
# Within this frame, users can configure which dots (7, 8, or both) are used to
# indicate selections, hyperlinks, and text attributes.
BRAILLE_INDICATORS = _("Indicators")

# Translators: This label for a widget from which the user can select which
# braille dot or dots should be used to indicate that character(s) being
# shown on the braille display are part of a link. The "indicator" can be chosen
# from among: None, Dot 7, Dot 8, Dots 7 and 8.
BRAILLE_HYPERLINK_INDICATOR = C_("Braille indicator", "Hyperlink:")

# Translators: This label for a widget from which the user can select which
# braille dot or dots should be used to indicate that character(s) being
# shown on the braille display are selected. The "indicator" can be chosen
# from among: None, Dot 7, Dot 8, Dots 7 and 8.
BRAILLE_SELECTION_INDICATOR = C_("Braille indicator", "Selection:")

# Translators: This label for a widget from which the user can select which
# braille dot or dots should be used to indicate that character(s) being
# shown on the braille display have a particular attribute (e.g. bold). The
# "indicator" can be chosen from among: None, Dot 7, Dot 8, Dots 7 and 8.
BRAILLE_TEXT_ATTRIBUTES_INDICATOR = C_("Braille indicator", "Text Attributes:")

# Translators: Braille flash messages are similar in nature to notifications or
# announcements. They are most commonly used for Orca to communicate
# Orca-specific information to the user via braille, such as confirming the
# toggling of an Orca setting via command. The reason they are called flash
# messages by screen readers is that they are shown on the refreshable braille
# display for only a brief time, after which the original contents of the
# display are restored. In instances where the message to be displayed is
# long/detailed, Orca provides a brief alternative. Users who prefer the brief
# alternative can uncheck this widget.
BRAILLE_MESSAGES_ARE_DETAILED = _("Messages are _detailed")

# Translators: Braille flash messages are similar in nature to notifications or
# announcements. They are most commonly used for Orca to communicate
# Orca-specific information to the user via braille, such as confirming the
# toggling of an Orca setting via command. The reason they are called flash
# messages by screen readers is that they are shown on the refreshable braille
# display for only a brief time, after which the original contents of the
# display are restored. Some users, however, would prefer to have the message
# remain displayed until they explicitly dismiss it. This can be accomplished
# by making flash messages persistent by checking this widget.
BRAILLE_MESSAGES_ARE_PERSISTENT = _("Messages are _persistent")

# Translators: This is the title of a section in the braille settings where the user
# can configure how Orca presents flash messages on a refreshable braille display.
BRAILLE_FLASH_MESSAGES = _("Flash Messages")

# Translators: This is the label for a setting for whether or not Orca should
# use abbreviated role names when presenting the role of an object on a
# refreshable braille display. For instance, if this option is enabled, Orca
# would present "btn" instead of "button".
BRAILLE_ABBREVIATED_ROLE_NAMES = _("_Abbreviated role names")

# Translators: This is the label for a setting for whether or not Orca should
# present contextual information about an object (e.g. the panel it is inside of)
# on a refreshable braille display.
BRAILLE_SHOW_CONTEXT = _("Show context (ancestors)")

# Translators: This is the label for a setting which turns on contracted braille.
# Contractions are shorter forms of commonly-used letter combinations and words.
# For instance in English there is a single braille symbol for "ing" in English
# braille.
BRAILLE_ENABLE_CONTRACTED_BRAILLE = _("Contracted Braille")

# Translators: This is the label for a setting which controls whether computer
# braille (uncontracted) is used for the word at the cursor location when
# contracted braille is enabled. If disabled, the word at the cursor remains
# contracted like the rest of the text.
BRAILLE_COMPUTER_BRAILLE_AT_CURSOR = _("Expand word at cursor")

# Translators: This is the title of the Orca Preferences dialog box.
DIALOG_SCREEN_READER_PREFERENCES = _("Screen Reader")

# Translators: This is the accessible name for the Orca Preferences dialog box.
DIALOG_SCREEN_READER_PREFERENCES_ACCESSIBLE = _("Screen Reader Preferences")

# Translators: This is the label for a widget in the preferences dialog.
DIALOG_ADD = _("Add")

# Translators: This is the label for a widget in the preferences dialog.
DIALOG_APPLY = _("Apply")

# Translators: This is the label for a widget in the preferences dialog.
DIALOG_CANCEL = _("Cancel")

# Translators: This is the label for a widget in the preferences dialog.
DIALOG_EDIT = _("Edit")

# Translators: This is the label for a widget in the preferences dialog.
DIALOG_HELP = _("Help")

# Translators: This is the label for a widget in the preferences dialog.
DIALOG_SAVE_AS = _("Save _As")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which determines whether or not key echo is enabled.
# If it is enabled, the user can then choose which types of keys they want to
# hear. See the strings which follow this one.
ECHO_ENABLE_KEY_ECHO = _("Key echo")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not alphabetic keys will
# be spoken when pressed.
ECHO_ALPHABETIC_KEYS = _("Alphabetic keys")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not numeric keys will
# be spoken when pressed.
ECHO_NUMERIC_KEYS = _("Numeric keys")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not punctuation keys will
# be spoken when pressed.
ECHO_PUNCTUATION_KEYS = _("Punctuation keys")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not function keys (F1, F2,
# etc.) will be spoken when pressed.
ECHO_FUNCTION_KEYS = _("Function keys")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not diacritical keys will
# be spoken when pressed.
ECHO_DIACRITICAL_KEYS = _("Diacritical keys")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not modifier keys (Shift,
# Control, Alt, etc.) will be spoken when pressed.
ECHO_MODIFIER_KEYS = _("Modifier keys")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not navigation keys (Arrows,
# Home, End, etc.) will be spoken when pressed.
ECHO_NAVIGATION_KEYS = _("Navigation keys")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not the space bar will
# be spoken when pressed.
ECHO_SPACE = _("Space")

# Translators: Orca has a feature to "echo" keys as they are pressed. This is
# the label for the setting which controls whether or not action keys such as
# Enter, Escape, Tab, Backspace, etc. will be spoken when pressed.
ECHO_ACTION_KEYS = _("Action keys")

# Translators: This is the label for a group of settings on the Echo preferences
# page. It groups the types of keys that will be spoken when pressed.
ECHO_KEYS_TO_ECHO = _("Keys to Echo")

# Translators: This is the label for a group of settings on the Echo preferences
# page. It groups the text echo options (character, word, sentence).
ECHO_TYPING_ECHO = _("Typing Echo")

# Translators: Orca has an "echo" feature to present text as it is being written
# by the user. While Orca's "key echo" options present the actual keyboard keys
# being pressed, "character echo" presents the character/string of length 1 that
# is inserted as a result of the keypress.
ECHO_CHARACTER = C_("Typing echo", "Character")

# Translators: Orca has an "echo" feature to present text as it is being written
# by the user. While Orca's "key echo" options present the actual keyboard keys
# being pressed, "sentence echo" presents the sentence that was just completed.
ECHO_SENTENCE = C_("Typing echo", "Sentence")

# Translators: Orca has an "echo" feature to present text as it is being written
# by the user. While Orca's "key echo" options present the actual keyboard keys
# being pressed, "word echo" presents the word that was just completed.
ECHO_WORD = C_("Typing echo", "Word")

# Translators: This is a label for a widget that allows the user to select which
# settings profile should be active. A profile is a collection of settings which
# can be saved and later loaded.
GENERAL_ACTIVE_PROFILE = _("Active _Profile:")

# Translators: Orca has a Say All feature which speaks the entire document.
# Some users want to hear additional information about what is being spoken. If
# this widget is enabled, Orca will announce that a form has been entered
# before speaking the contents of that form. At the end of the form, Orca will
# announce that the form is being exited.
GENERAL_ANNOUNCE_FORMS_IN_SAY_ALL = _("Announce _forms in Say All")

# Translators: Orca has a Say All feature which speaks the entire document.
# Some users want to hear additional information about what is being spoken. If
# this widget is enabled, Orca will announce that a panel has been entered
# before speaking the new location. At the end of the panel contents, Orca will
# announce that the panel is being exited. A panel is a generic container of
# objects, such as a group of related form fields.
GENERAL_ANNOUNCE_PANELS_IN_SAY_ALL = _("Announce _panels in Say All")

# Translators: Orca has a Say All feature which speaks the entire document.
# Some users want to hear additional information about what is being spoken. If
# this widget is enabled, Orca will announce that a table with x rows and y
# columns has been entered before speaking the content of that table. At the
# end of the table content, Orca will announce that the table is being exited.
GENERAL_ANNOUNCE_TABLES_IN_SAY_ALL = _("Announce _tables in Say All")

# Translators: Orca has a Say All feature which speaks the entire document.
# Some users want to hear additional information about what is being spoken. If
# this widget is enabled, Orca will announce that a blockquote has been
# entered before speaking the text. At the end of the text, Orca will announce
# that the blockquote is being exited.
GENERAL_ANNOUNCE_BLOCKQUOTES_IN_SAY_ALL = _("Announce block_quotes in Say All")

# Translators: Orca has a Say All feature which speaks the entire document.
# Some users want to hear additional information about what is being spoken. If
# this widget is enabled, Orca will announce when an ARIA landmark has been
# entered or exited. ARIA landmarks are the W3C defined HTML tag attribute
# 'role' used to identify important part of webpage like banners, main context,
# search, etc.
GENERAL_ANNOUNCE_LANDMARKS_IN_SAY_ALL = _("Announce land_marks in Say All")

# Translators: Orca has a Say All feature which speaks the entire document.
# Some users want to hear additional information about what is being spoken. If
# this widget is enabled, Orca will announce that a list with x items has
# been entered before speaking the content of that list. At the end of the list
# content, Orca will announce that the list is being exited.
GENERAL_ANNOUNCE_LISTS_IN_SAY_ALL = _("Announce li_sts in Say All")

# Translators: Orca has a setting which determines which progress bar updates
# should be announced. The options are all progress bars, only progress bars in
# the active application, or only progress bars in the current window.
GENERAL_APPLIES_TO = _("Applies to:")

# Translators: Orca has a setting which determines which progress bar updates
# should be announced. The options are all progress bars, only progress bars in
# the active application, or only progress bars in the current window. This
# label is for the second option.
GENERAL_APPLICATION = _("Application")

# Translators: Orca has a setting which determines which progress bar updates
# should be announced. The options are all progress bars, only progress bars in
# the active application, or only progress bars in the current window. This
# label is for the third option.
GENERAL_WINDOW = _("Window")

# Translators: This is an option in the Preferences dialog box related to the
# presentation of progress bar updates. If this widget is enabled, Orca will
# periodically emit beeps which increase in pitch as the value of the progress
# bar increases.
GENERAL_BEEP_UPDATES = _("Bee_p updates")

# Translators: This is a label for the setting which controls how Orca will
# present the date.
GENERAL_DATE_FORMAT = _("Dat_e format:")

# Translators: This is a label for the setting which controls how Orca will
# present the time.
GENERAL_TIME_FORMAT = _("_Time format:")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text. The
# following string is the label for the profiles sidebar item.
GENERAL_PROFILES = _("Profiles")

# Translators: This is the heading shown at the top of the profiles preferences
# page, indicating which profile is currently selected and being edited.
CURRENT_PROFILE = _("Current Profile")

# Translators: This is an informational message shown at the top of the Profiles
# preferences page. It explains to the user what profiles are and how they work.
# The quoted profile name is `PROFILE_DEFAULT = C_("Profile", "Default")` defined
# earlier in this file.
PROFILES_INFO = (
    _(
        'Profiles are collections of settings. The "%s" profile is '
        "loaded at startup. To edit an existing profile, select it below and then "
        "adjust settings on the other pages. To rename or delete the selected profile, "
        "use the Tab key to navigate to its associated buttons.",
    )
    % PROFILE_DEFAULT
)

# Translators: This is a label in the Preferences dialog box. It applies to
# several options related to which progress bars Orca should speak and how
# often Orca should speak them.
PROGRESS_BARS = _("Progress Bars")

# Translators: Orca has a Say All feature which speaks the entire document.
# This is the label for the group of settings related to Say All.
GENERAL_SAY_ALL = _("Say All")

# Translators: Say all by refers to the way that Orca will say (speak) an
# amount of text -- in particular, where Orca where insert pauses. There are
# currently two choices (supplied by a combo box to the right of this label):
# say all by sentence and say all by line.  If Orca were speaking a work of
# fiction, it would probably be best to do say all by sentence so it sounds
# more natural. If Orca were speaking something like a page of computer
# commands, doing a say all by line would work better.
SAY_ALL_BY = _("Say All By")

# Translators: This is an informational message on the Say All preferences page
# explaining what Say All does. Please translate this message consistent with the
# strings in `GENERAL_SAY_ALL = _("Say All")` and SAY_ALL_BY = _("Say All By")`
SAY_ALL_INFO = _(
    "Say All reads from the current location to the end of the document, "
    'pausing after each sentence or line based on the "Say All By" value.',
)

# Translators: This is an informational message on the Say All preferences page
# explaining the options found under the section with the following label:
# `SAY_ALL_REWIND_AND_FAST_FORWARD_BY = _("Rewind and Fast Forward By")`
# Please translate "structural navigation" using terminology consistent with
# `KB_GROUP_STRUCTURAL_NAVIGATION = _("Structural navigation")`
SAY_ALL_NAVIGATION_INFO = _(
    "If enabled, you can use the arrow keys and/or structural navigation commands "
    "to move within the document while Say All is active.",
)

# Translators: This is an informational message on the Say All preferences page
# explaining the options found under the section with the following label:
# ANNOUNCEMENTS = _("Container Announcements")
SAY_ALL_CONTAINER_INFO = _(
    'The "Container Announcements" settings determine what additional details '
    "will be announced as Say All moves through the document.",
)

GENERAL_SPEAK_OBJECT_UNDER_MOUSE = _("Speak object under mo_use")

# Translators: Profiles in Orca make it possible for users to quickly switch
# amongst a group of pre-defined settings (e.g. an 'English' profile for reading
# text written in English using an English-language speech synthesizer and
# braille rules, and a similar 'Spanish' profile for reading Spanish text. The
# following string is the label for the widget from which the user can select
# which profile should be loaded when Orca starts up.
GENERAL_START_UP_PROFILE = _("Start-up Profile:")

# Translators: This is an option in the Preferences dialog box related to the
# presentation of progress bar updates. If this widget is enabled, Orca will
# periodically display the current percentage in braille.
GENERAL_BRAILLE_UPDATES = _("_Braille updates")


# This button will load the selected settings profile in the application.
GENERAL_LOAD = _("_Load")

GENERAL_PRESENT_TOOLTIPS = _("_Present tooltips")

# Translators: This is the label for a button in a dialog.
GENERAL_REMOVE = _("_Remove")

# Translators: This is an option in the Preferences dialog box related to the
# presentation of progress bar updates. If this widget is enabled, Orca will
# periodically speak the current percentage.
GENERAL_SPEAK_UPDATES = _("_Speak updates")

# Translators: Here this is a label for a spin button through which a user can
# customize the frequency in seconds an announcement should be made regarding
# the current value of a progress bar.
GENERAL_FREQUENCY_SECS = C_("ProgressBar", "Frequency (secs):")

# Translators: The "Orca modifier" is a key that Orca uses for its own commands.
# The default Orca modifier is KP_Insert for the "desktop" keyboard layout and
# Caps Lock for the "laptop" keyboard layout. This string is a label for choosing
# which key(s) should be used as the Orca modifier.
KEY_BINDINGS_SCREEN_READER_MODIFIER_KEY_S = _("Screen Reader _Modifier Key(s):")

# Translators: Orca can optionally speak additional details as the user
# navigates (e.g. via the arrow keys) within document content. If this widget
# is enabled, Orca will announce that a form has been entered as the user
# arrows into it and before speaking the new location. Upon navigating out of
# the form, Orca will announce that the form has been exited prior to speaking
# the new location.
SPEECH_ANNOUNCE_FORMS_DURING_NAVIGATION = _("Announce _forms during navigation")

# Translators: Orca can optionally speak additional details as the user
# navigates (e.g. via the arrow keys) within document content.  If this
# widget is enabled, Orca will announce that a list with x items has been
# entered as the user arrows into it and before speaking the list content. Upon
# navigating out of the list, Orca will announce that the list has been exited
# prior to speaking the new location.
SPEECH_ANNOUNCE_LISTS_DURING_NAVIGATION = _("Announce _lists during navigation")

# Translators: Orca can optionally speak additional details as the user
# navigates (e.g. via the arrow keys) within document content.  If this
# widget is enabled, Orca will announce that a panel has been entered as the
# user arrows into it and before speaking the new location. Upon navigating out
# of the panel, Orca will announce that the panel has been exited prior to
# speaking the new location. A panel is a generic container of objects, such as
# a group of related form fields.
SPEECH_ANNOUNCE_PANELS_DURING_NAVIGATION = _("Announce _panels during navigation")

# Translators: Orca can optionally speak additional details as the user
# navigates (e.g. via the arrow keys) within document content.  If this
# widget is enabled, Orca will announce that a table with x rows and y
# columns has been entered as the user arrows into it and before speaking the
# table content. Upon navigating out of the table, Orca will announce that the
# table has been exited prior to speaking the new location.
SPEECH_ANNOUNCE_TABLES_DURING_NAVIGATION = _("Announce _tables during navigation")

# Translators: Orca can optionally speak additional details as the user
# navigates (e.g. via the arrow keys) within document content.  If this
# widget is enabled, Orca will announce that a blockquote has been entered as
# the user arrows into it and before speaking the text. Upon navigating out of
# the blockquote, Orca will announce that the blockquote has been exited prior
# to speaking the new location.
SPEECH_ANNOUNCE_BLOCKQUOTES_DURING_NAVIGATION = _("Announce block_quotes during navigation")

# Translators: Orca can optionally speak additional details as the user
# navigates (e.g. via the arrow keys) within document content.  If this
# widget is enabled, Orca will announce the ARIA landmark that has been
# entered as the user arrows into it and before speaking the text. Upon
# navigating out of the landmark, Orca will announce that the landmark has been
# exited prior to speaking the new location. ARIA landmarks are the W3C defined
# HTML tag attribute 'role' used to identify important part of webpage like
# banners, main context, search, etc.
SPEECH_ANNOUNCE_LANDMARKS_DURING_NAVIGATION = _("Announce land_marks during navigation")

# Translators: If this setting is enabled, Orca will only speak text which is
# actually displayed on the screen. It will NOT speak things like the role of
# an item (e.g. widget) or its state (e.g. not checked) or say misspelled to
# indicate the presence of red squiggly spelling error lines -- things which
# Orca normally speaks. This setting is primarily intended for low vision users
# and sighted users with a learning disability.
SPEECH_ONLY_SPEAK_DISPLAYED_TEXT = _("Only speak displayed text")

# Translators: Orca has a command to present font and formatting information,
# including foreground and background color. The setting associated with this
# widget determines how Orca will speak colors: As rgb values or as names
# (e.g. light blue).
SPEECH_SPEAK_COLORS_AS_NAMES = _("S_peak colors as names")

# Translators: This is a label for a widget associated with whether Orca will
# present the mnemonic (underlined character) of an object, such as a button or
# menu item, when it becomes focused/selected.
PRESENT_OBJECT_MNEMONICS = _("Mnemonics")

# Translators: If this widget is enabled, Orca will speak the accessible
# description of an object. Whereas the accessible name of an object tends to
# be short and typically corresponds to what is displayed on screen, the
# contents of the accessible description tend to be longer, e.g. matching the
# text of the tooltip, and are sometimes redundant to the accessible name.
# Therefore, we allow the user to opt out of this additional information.
SPEECH_SPEAK_DESCRIPTION = _("Description")

# Translators: This is a label for a widget associated with whether Orca will
# speak indentation and justification information for text content.
SPEECH_SPEAK_INDENTATION_AND_JUSTIFICATION = _("Indentation and justification")

# Translators: This is the label for a widget on the Speech preferences page.
# When checked, indentation and justification will only be announced when they
# have changed from the previous line.
SPEECH_INDENTATION_ONLY_IF_CHANGED = _("Only speak indentation if changed")

# Translators: The misspelled-word indicator is the red squiggly line that
# appears underneath misspelled words in editable text fields. If this setting
# is enabled, when a user first moves into a word with this indicator, or types
# a misspelled word causing this indicator to appear, Orca will announce that
# the word is misspelled.
SPEECH_SPEAK_MISSPELLED_WORD_INDICATOR = _("Misspelled-word indicator")

# Translators: This is a label for a widget associated with whether Orca will
# speak blank lines when navigating in document content.
SPEECH_SPEAK_BLANK_LINES = _("Blank lines")

# Translators: This widget toggles whether or not Orca says the child
# position (e.g., item 6 of 7).
SPEECH_SPEAK_CHILD_POSITION = _("Position in set")

# Translators: This widget is associated with the setting that determines
# what happens if a user presses Up or Down arrow to move row by row in a GUI
# table, such as a GtkTreeView. Document tables, such as those found in Writer
# and web content, and spreadsheet tables such as those found in Calc are not
# considered GUI tables. If this setting is enabled, Orca will speak the entire
# row; if it is disabled, Orca will only speak the cell with focus.
SPEECH_SPEAK_FULL_ROW_IN_GUI_TABLES = _("Speak full row in _GUI tables")

# Translators: This widget is associated with the setting that determines
# what happens if a user presses Up or Down arrow to move row by row in a
# document table. In this context, document tables include tables such as those
# found in Writer documents as well as HTML table elements, but exclude
# spreadsheet tables such as found in Calc. If this setting is enabled, Orca
# will speak the entire row; if it is disabled, Orca will only speak the cell
# with focus.
SPEECH_SPEAK_FULL_ROW_IN_DOCUMENT_TABLES = _("Speak full row in _document tables")

# Translators: This widget is associated with the setting that determines
# what happens if a user presses Up or Down arrow to move row by row in a
# spreadsheet. If this setting is enabled, Orca will speak the entire row; if
# it is disabled, Orca will only speak the cell with focus.
SPEECH_SPEAK_FULL_ROW_IN_SPREADSHEETS = _("Speak full row in sp_readsheets")

# Translators: This is a label for a widget associated with whether Orca speaks
# tutorial messages / "help text".
SPEECH_SPEAK_TUTORIAL_MESSAGES = _("Tutorial messages")

# Translators: This is the label for a group of settings on the Speech
# preferences page. It groups settings related to how objects are presented
# via speech (descriptions, mnemonics, child position).
SPEECH_OBJECT_DETAILS = _("Spoken Object Details")

# Translators: This is a setting in the Speech and Braille preferences pages.
# When enabled, Orca will provide detailed/verbose object descriptions.
OBJECT_PRESENTATION_IS_DETAILED = _("Object presentation is detailed")

# Translators: This is the label for a checkbox on the Speech preferences page.
# When checked, Orca will apply the user's pronunciation dictionary when speaking.
# The pronunciation dictionary allows users to customize how specific words or
# character sequences are spoken by the speech synthesizer.
SPEECH_USE_PRONUNCIATION_DICTIONARY = _("Use pronunciation dictionary")

# Translators: This is a label for a group of widgets from which the user can
# chose what is and is not spoken by Orca during navigation or Say All. For
# instance, when the user first navigates into a table, should Orca speak the
# table details or just the newly focused/selected cell?
SPEECH_SPOKEN_CONTEXT = _("Spoken Context")

# Translators: This is the label for a widget in the preferences dialog.
# If selected speech support will be enabled.
SPEECH_ENABLE_SPEECH = _("Speech support")

# Translators: Orca has system messages which are similar in nature to
# notifications or announcements. They are most commonly used for Orca to
# communicate Orca-specific information to the user via speech, such as
# confirming the toggling of an Orca setting via command.  In instances where
# the message to be displayed is long/detailed, Orca provides a brief
# alternative. Users who prefer that brief alternative can uncheck this
# widget.
SPEECH_SYSTEM_MESSAGES_ARE_DETAILED = _("_System messages are detailed")

# Translators: The on-screen braille display is a graphical window that displays a
# visual representation of what would be shown on a connected braille display,
# including the braille dots and the text characters they correspond to. It is
# primarily used by developers and testers who may not have a physical braille
# display.
BRAILLE_MONITOR = _("On-screen braille")

# Translators: The on-screen speech display is a graphical window that displays a
# scrolling log of all text spoken by the screen reader. It is primarily used by
# developers and testers to verify what is being spoken without having to listen
# to the audio.
SPEECH_MONITOR = _("On-screen speech")

# Translators: This is the title of the on-screen display settings category in the
# Orca Preferences dialog. These settings control the appearance of the on-screen
# braille and speech monitor windows.
ON_SCREEN_DISPLAY = _("On-Screen Display")

# Translators: This is the label of a spin button that controls the number of braille
# cells displayed in the on-screen braille monitor.
BRAILLE_MONITOR_CELL_COUNT = _("Cell count")

# Translators: This is the label of a checkbox that controls whether the on-screen
# braille monitor shows Unicode braille dot patterns instead of text characters.
BRAILLE_MONITOR_SHOW_DOTS = _("Show braille dot patterns")

# Translators: This is the label of a color button that controls the text color
# in the on-screen braille monitor.
BRAILLE_MONITOR_FOREGROUND = _("Text color")

# Translators: This is the label of a color button that controls the background color
# in the on-screen braille monitor.
BRAILLE_MONITOR_BACKGROUND = _("Background color")

# Translators: This is the label of a spin button that controls the font size of
# text in the on-screen speech monitor.
SPEECH_MONITOR_FONT_SIZE = _("Font size")

# Translators: This is the label of a color button that controls the text color
# in the on-screen speech monitor.
SPEECH_MONITOR_FOREGROUND = _("Text color")

# Translators: This is the label of a color button that controls the background color
# in the on-screen speech monitor.
SPEECH_MONITOR_BACKGROUND = _("Background color")

# Translators: This is an informational message displayed at the top of the on-screen
# speech display preferences page, briefly explaining the feature and how to toggle it.
SPEECH_MONITOR_INFO = _(
    "The on-screen speech display is a window that shows what the screen reader "
    "is speaking. The default keybinding to show or hide it is Orca+Shift+d. "
    "This can be changed in Commands.",
)

# Translators: This is an informational message displayed at the top of the on-screen
# braille display preferences page, briefly explaining the feature and how to toggle it.
BRAILLE_MONITOR_INFO = _(
    "The on-screen braille display is a window intended for developers that shows "
    "what would be presented on a braille display. To show or hide it, assign a "
    "keybinding in Commands.",
)

# Translators: This is the label of the Braille page in the Orca Preferences dialog.
BRAILLE = _("Braille")

# Translators: This is the label of the Documents page in the Orca Preferences
# dialog. On that page there are settings related to navigating and reading documents,
# including web pages, word processor documents, and PDFs.
DOCUMENTS = _("Documents")

# Translators: This string is a label for a preferences page containing settings
# that apply when the user is navigating using the application's native navigation
# (e.g., arrow keys handled by the app) rather than the screen reader's browse mode.
NATIVE_NAVIGATION = _("Native navigation")

# Translators: This string is a label for a group of settings related to what
# happens when a document or web page finishes loading.
PAGE_LOAD = _("Page load")

# Translators: This is the label for a group of settings related to what Orca
# announces when entering/exiting different types of containers/ancestors.
ANNOUNCEMENTS = _("Container Announcements")

# Translators: This is a heading for a group of toggleable widgets. Each one controls
# whether Orca announces entering a type of container (e.g. blockquotes, forms,
# landmarks, lists, panels, tables). The full meaning is "Announce when entering
# [container type]" where the container types are listed below this heading.
ANNOUNCE_WHEN_ENTERING = _("Announce when entering")

# Translators: This is the label of the Echo page in the Orca Preferences dialog.
# On that page there are a variety of settings related to what Orca will echo
# as the user types on the keyboard.
ECHO = _("Echo")

# Translators: This text appears at the top of the Echo preferences page. It explains
# the difference between key echo (speaking the key pressed) and typing echo (speaking
# what was inserted as a result of typing).
ECHO_INFO = _(
    "Key echo speaks the key you pressed. Typing echo speaks what was inserted "
    "into the document as a result of your typing.",
)

# Translators: This is the label of the General page in the Orca Preferences
# dialog. On that page there are a variety of general Orca settings which do
# not fit well into any of the other pages.
GENERAL = _("General")

# Translators: This is the label of the Key Bindings page in the Orca Preferences
# dialog. On that page there is a list of all Orca commands and the keystrokes
# associated with them. The user can customize the keystrokes for any command.
KEY_BINDINGS = _("Key Bindings")

# Translators: This is the label of the Commands page in the Orca Preferences
# dialog. On that page there is a list of all Orca commands and the keystrokes
# associated with them. The user can customize the keystrokes for any command.
COMMANDS = _("Commands")

# Translators: This is the label of the Mouse page in the Orca Preferences dialog.
# On that page there are settings related to what information Orca presents when
# the mouse pointer moves.
MOUSE = _("Mouse")

# Translators: This is an informational message displayed on Orca's Mouse
# preferences page. Mouse review features may not work properly on
# Wayland because Wayland restricts applications from monitoring the
# mouse pointer.
MOUSE_WAYLAND_WARNING = _("These settings may not work on Wayland.")

# Translators: This is the label of the Pronunciation page in the Orca Preferences
# dialog. On that page there is UI for customizing how a given word will be sent
# to the speech synthesizer. For instance "idk" can be sent to the speech server
# as "I don't know" or "I D K" or "eye dee kay" or whatever causes the user's
# speech synthesizer to say what the user finds most helpful.
PRONUNCIATION = _("Pronunciation")

# Translators: This is the label of the Sound page in the Orca Preferences dialog.
# On that page there are settings related to sound output, including volume
# and progress bar beep notifications.
SOUND = _("Sound")

# Translators: This is the label for a widget in the preferences dialog which
# turns sound support on or off.
SOUND_ENABLE_SOUND_SUPPORT = _("Sound support")

# Translators: This is the label for the volume control in the Sound preferences.
SOUND_VOLUME = _("Volume")

# Translators: This is the label of the Tables page in the Orca Preferences
# dialog. On that page there are settings related to table navigation, such as
# whether to speak table cell coordinates, row and column headers, and whether
# to read tables cell by cell or by full rows.
TABLES = _("Tables")

# Translators: This is the label of the Time and Date page in the Orca Preferences
# dialog.
TIME_AND_DATE = _("Time and Date")

# Translators: This is the label of the Speech page in the Orca Preferences dialog.
SPEECH = _("Speech")

# Translators: This is the label of the Text Attributes page in the Orca Preferences
# dialog.
TEXT_ATTRIBUTES = _("Text Attributes")

# Translators: This text appears at the top of the Text Attributes preferences
# page. It explains that users can configure which text formatting attributes
# (such as bold, italic, underline) should be announced via speech and/or braille,
# and that the order of the attributes controls the order in which they are presented.
TEXT_ATTRIBUTES_INFO = _(
    "Configure which text attributes are spoken and/or marked in braille, "
    "and the order in which they are presented.",
)

# Translators: This is the label of the Voice page in the Orca Preferences
# dialog.
VOICE = _("Voice")

# Translators: This is the label for a group of settings on the Voice preferences
# page. It contains widgets that control whether speech is enabled and how
# certain content (numbers, colors) is spoken.
VOICE_SPEECH_SETTINGS = _("Speech Settings")

# Translators: This is the accessible label for a button in the Voice preferences
# that opens a dialog to configure settings for a specific voice type (such as
# default, hyperlink, uppercase, or system). The button displays only an icon
# (a cog/gear), so this label is for screen reader users. The %s is replaced
# with the voice type name (e.g., "Default", "Uppercase", "Hyperlink", "System").
VOICE_TYPE_SETTINGS = C_("VoiceType", "%s Settings")

# Translators: This is a label for a setting that controls whether Orca will
# automatically switch the speech synthesizer's language based on the language
# of the text or UI element being spoken.
AUTO_LANGUAGE_SWITCHING = _("Automatic language switching")

# Translators: This label is for a group of buttons on the Text Attributes
# pane of the Orca Preferences dialog. On that pane there is a long list of
# possible text attributes. The user can select one and then, by using the
# Move buttons in this group, adjust when that attribute is spoken by Orca.
TEXT_ATTRIBUTES_ADJUST_SELECTED_ATTRIBUTE = _("Adjust selected attribute")

# Translators: This label for a widget from which the user can select which
# braille dot or dots should be used to indicate that character(s) being
# shown on the braille display have a particular attribute (e.g. bold). The
# "indicator" can be chosen from among: None, Dot 7, Dot 8, Dots 7 and 8.
TEXT_ATTRIBUTES_BRAILLE_INDICATOR = _("Braille Indicator")

# Translators: This label is on a button on the Text Attributes pane of the
# Orca Preferences dialog. On that pane there is a long list of possible text
# attributes. The user can select one and then, by using the Move _down one
# button, move that attribute down one line in the list. The ordering in the
# list is important as Orca will speak the selected text attributes in the
# given order.
TEXT_ATTRIBUTES_MOVE_DOWN_ONE = _("Move down one")

# Translators: This label is on a button on the Text Attributes pane of the
# Orca Preferences dialog. On that pane there is a long list of possible text
# attributes. The user can select one and then, by using the Move _up one
# button, move that attribute up one line in the list. The ordering in the list
# is important as Orca will speak the selected text attributes in the given
# order.
TEXT_ATTRIBUTES_MOVE_UP_ONE = _("Move up one")

# Translators: This label is on a button on the Text Attributes pane of the
# Orca Preferences dialog. On that pane there is a long list of possible text
# attributes. The user can select one and then, by using the Move to _bottom
# button, move that attribute to the bottom of the list. The ordering in the
# list is important as Orca will speak the selected text attributes in the
# given order.
TEXT_ATTRIBUTES_MOVE_TO_BOTTOM = _("Move to bottom")

# Translators:  This label is on a button on the Text Attributes pane of the
# Orca Preferences dialog. On that pane there is a long list of possible text
# attributes. The user can select one and then, by using the Move to _top
# button, move that attribute to the top of the list. The ordering in the list
# is important as Orca will speak the selected text attributes in the given
# order.
TEXT_ATTRIBUTES_MOVE_TO_TOP = _("Move to top")

# Translators: This is a label for a group of widgets associated with how Orca
# will present text attributes such as bold, underline, italic, font size,
TEXT_ATTRIBUTES_TEXT_ATTRIBUTES = _("Text attributes")

# Translators: This is the accessible name for a widget in the preferences dialog
# that allows the user to reorder text attributes in the list.
TEXT_ATTRIBUTES_REORDER = _("Reorder")

# Translators: This refers to the amount of information Orca provides about a
# particular object that receives focus. The choices are Brief and Verbose.
VERBOSITY = _("Verbosity")

# Translators: This refers to the amount of information Orca provides about a
# particular object that receives focus.
VERBOSITY_LEVEL_VERBOSE = _("Ver_bose")

# Translators: This is a label for a group of widgets associated with how Orca
# will speak regardless of which voice (Default, Hyperlink, System, Uppercase)
# is being used. For instance, Orca can speak numbers as individual digits
# (e.g. 123 as 1 2 3) or as a whole (e.g. 123 as one hundred and twenty three).
VOICE_GLOBAL_VOICE_SETTINGS = _("Global Voice Settings")

# Translators: If this setting is enabled, 123 will be spoken as the individual
# digits 1 2 3; otherwise, it will be sent to the synthesizer and (likely)
# spoken as one hundred and twenty three.
VOICE_SPEAK_NUMBERS_AS_DIGITS = _("Speak _numbers as digits")

# Translators: Having multiple voice types in Orca makes it possible for the
# user to more quickly identify properties of text non-visually, such as the
# fact that text is written in capital letters or is a link; or that text is
# actually visible on the screen as opposed to an Orca-specific message. The
# available voice types in Orca include: default, uppercase, hyperlink, and
# system -- each of which can be configured by the user to sound the way he/she
# finds most helpful. This string is displayed in the label for the group of
# all of the controls associated with configuring a particular voice type.
VOICE_VOICE_TYPE_SETTINGS = _("Voice Type Settings")

# Translators: This is the label for a widget from which the user can select
# which speech synthesis system Orca should use. Examples of speech synthesis
# systems include Speech Dispatcher and Spiel.
VOICE_SPEECH_SYSTEM = _("Speech _system:")

# Translators: This is the label for a widget from which the user can select
# which speech synthesizer Orca should use. Examples of speech synthesizers
# include eSpeak, Festival, and Pico.
VOICE_SPEECH_SYNTHESIZER = _("Speech synthesi_zer:")

# Translators: Orca uses Speech Dispatcher to present content to users via
# text-to-speech. Speech Dispatcher has a feature to control how capital
# letters are presented: Do nothing at all; say the word 'capital' prior to
# presenting a capital letter (which Speech Dispatcher refers to as 'spell'),
# or play a tone (which Speech Dispatcher refers to as a sound 'icon'). Orca
# refers to these things as 'capitalization style'. This string is the text of
# the label through which users can choose which of style they would prefer.
VOICE_CAPITALIZATION_STYLE = _("Capitalization style")

# Translators: This is the label for a widget from which the user can select
# the language of the speech synthesizer.
VOICE_LANGUAGE = _("_Language:")

# Translators: This is the label for a widget from which the user can select
# which voice from the current speech synthesizer should be used. Often voices
# have human names such as "Allison". Hence the use of the term person.
VOICE_PERSON = _("_Person:")

# Translators: This is the label for a widget from which the user can set the
# speaking rate of the current voice.
VOICE_RATE = _("_Rate:")

# Translators: This is the label for a widget from which the user can set the
# pitch of the current voice.
VOICE_PITCH = _("Pi_tch:")

# Translators: This is the label for a widget from which the user can set the
# volume of the current voice.
VOICE_VOLUME = _("Vo_lume:")

# Translators: Having multiple voice types in Orca makes it possible for the
# user to more quickly identify properties of text non-visually, such as the
# fact that text is written in capital letters or is a link; or that text is
# actually visible on the screen as opposed to an Orca-specific message. The
# available voice types in Orca include: default, uppercase, hyperlink, and
# system -- each of which can be configured by the user to sound the way he/she
# finds most helpful. This string is displayed in the label for the combo box
# in which the user selects a voice type to configure.
VOICE_VOICE_TYPE = _("_Voice type:")

# Translators: This refers to the voice used by Orca by default.
VOICE_TYPE_DEFAULT = _("Default")

# Translators: This refers to the voice used by Orca when presenting one or more
# characters which are part of a hyperlink.
VOICE_TYPE_HYPERLINK = _("Hyperlink")

# Translators: This refers to the voice used by Orca when presenting information
# which is not displayed on the screen as text, but is still being communicated
# by the system in some visual fashion. For instance, Orca says "misspelled" to
# indicate the presence of the red squiggly line found under a spelling error;
# Orca might say "3 of 6" when a user Tabs into a list of six items and the
# third item is selected. And so on.
VOICE_TYPE_SYSTEM = _("System")

# Translators: This refers to the voice used by Orca when presenting one or more
# characters which is written in uppercase.
VOICE_TYPE_UPPERCASE = _("Uppercase")


def notifications_count(count):
    """Returns the gui label representing the notifications count."""

    # Translators: Orca has a feature to list all of the notification messages
    # received, similar to the functionality gnome-shell provides when you press
    # Super+M, but it works in all desktop environments. This string is the title
    # of the dialog that contains the list of notification messages. The string
    # substitution is for the number of messages in the list.
    return ngettext("%d notification", "%d notifications", count) % count



# Orca
#
# Copyright 2005-2008 Sun Microsystems Inc.
# Copyright 2011-2016 Igalia, S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=too-many-public-methods
# pylint: disable=too-many-instance-attributes

"""Provides support for handling input events."""

from __future__ import annotations

import inspect
import math
import time
import unicodedata
from typing import TYPE_CHECKING

import gi

gi.require_version("Atspi", "2.0")
gi.require_version("Gdk", "3.0")
from gi.repository import (
    Atspi,
    Gdk,  # pylint: disable=no-name-in-module
    GLib,
)

from . import (
    command_manager,
    debug,
    focus_manager,
    keybindings,
    keynames,
    messages,
    orca_modifier_manager,
    presentation_manager,
    script_manager,
)
from .ax_utilities import AXUtilities

if TYPE_CHECKING:
    from collections.abc import Callable

    from .scripts import default

KEYBOARD_EVENT = "keyboard"
BRAILLE_EVENT = "braille"
MOUSE_BUTTON_EVENT = "mouse:button"
REMOTE_CONTROLLER_EVENT = "remote controller"


class InputEvent:
    """Provides support for handling input events."""

    def __init__(self, event_type: str) -> None:
        """Creates a new KEYBOARD_EVENT, BRAILLE_EVENT, or MOUSE_BUTTON_EVENT."""

        self.type: str = event_type
        self.time: float = time.time()
        self._click_count: int = 0

    def get_click_count(self) -> int:
        """Return the count of the number of clicks a user has made."""

        return self._click_count

    def set_click_count(self, count: int) -> None:
        """Updates the count of the number of clicks a user has made."""

        self._click_count = count

    def as_single_line_string(self) -> str:
        """Returns a single-line string representation of this event."""

        return f"{self.type}"


class KeyboardEvent(InputEvent):
    """Provides support for handling keyboard events."""

    # pylint:disable=too-many-arguments
    # pylint:disable=too-many-positional-arguments
    def __init__(self, pressed: bool, keycode: int, keysym: int, modifiers: int, text: str) -> None:
        """Creates a new InputEvent of type KEYBOARD_EVENT.

        Arguments:
        - pressed: True if this is a key press, False for a release.
        - keycode: the hardware keycode.
        - keysym: the translated keysym.
        - modifiers: a bitflag giving the active modifiers.
        - text: the text that would be inserted if this key is pressed.
        """

        super().__init__(KEYBOARD_EVENT)
        self.id: int = keysym
        self.type: Atspi.EventType = (
            Atspi.EventType.KEY_PRESSED_EVENT if pressed else Atspi.EventType.KEY_RELEASED_EVENT
        )
        self.hw_code: int = keycode
        self._text: str = text
        self.modifiers: int = modifiers & Gdk.ModifierType.MODIFIER_MASK  # pylint: disable=no-member
        if modifiers & (1 << Atspi.ModifierType.NUMLOCK):
            self.modifiers |= 1 << Atspi.ModifierType.NUMLOCK
        self.keyval_name: str = Gdk.keyval_name(keysym) or ""
        self.timestamp: float = time.time()
        self._script: default.Script | None = None
        self._window: Atspi.Accessible | None = None
        self._obj: Atspi.Accessible | None = None
        self._handler: Callable[[], bool] | None = None
        self._is_kp_with_numlock: bool = False

        # Some implementors don't include numlock in the modifiers. Unfortunately,
        # trying to heuristically hack around this just by looking at the event
        # is not reliable. Ditto regarding asking Gdk for the numlock state.
        if self._is_keypad_key() and self.modifiers & (1 << Atspi.ModifierType.NUMLOCK):
            self._is_kp_with_numlock = True

        modifier_manager = orca_modifier_manager.get_manager()
        if self.is_orca_modifier():
            modifier_manager.set_pressed_state(pressed)
        if modifier_manager.get_pressed_state():
            self.modifiers |= keybindings.ORCA_MODIFIER_MASK

    # pylint:enable=too-many-arguments
    # pylint:enable=too-many-positional-arguments

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, KeyboardEvent):
            return False

        if self.type == other.type and self.hw_code == other.hw_code:
            return self.timestamp == other.timestamp

        return False

    def __hash__(self) -> int:
        return hash((self.type, self.hw_code, self.timestamp))

    def __str__(self) -> str:
        if self.should_obscure():
            keyid = hw_code = modifiers = text = keyval_name = "*"
        else:
            keyid = str(self.id)
            hw_code = str(self.hw_code)
            modifiers = str(self.modifiers)
            keyval_name = self.keyval_name
            text = self._text

        return (
            f"KEYBOARD_EVENT:  type={self.type.value_name.upper()}\n"
            f"                 id={keyid}\n"
            f"                 hw_code={hw_code}\n"
            f"                 modifiers={modifiers}\n"
            f"                 text='{text}'\n"
            f"                 keyval_name='{keyval_name}'\n"
            f"                 timestamp={self.timestamp}\n"
            f"                 clickCount={self._click_count}"
        )

    def as_single_line_string(self) -> str:
        """Returns a single-line string representation of this event."""

        if self.should_obscure():
            return "(obscured)"

        return (
            f"'{self.keyval_name}' ({self.hw_code}) mods: {self.modifiers} {self.type.value_nick}"
        )

    def is_alt_control_or_orca_modified(self) -> bool:
        """Return True if this key is Alt, Control, or Orca modified."""

        return bool(
            self.modifiers & keybindings.CTRL_MODIFIER_MASK
            or self.modifiers & keybindings.ALT_MODIFIER_MASK
            or self.modifiers & keybindings.ORCA_MODIFIER_MASK,
        )

    def should_obscure(self) -> bool:
        """Returns True if we should obscure the details of this event."""

        if not AXUtilities.is_password_text(self._obj):
            return False

        if not self.is_printable_key() and not self.is_space():
            return False

        return not self.is_alt_control_or_orca_modified()

    def is_navigation_key(self) -> bool:
        """Return True if this is a navigation key."""

        keys = [
            Gdk.KEY_Down,
            Gdk.KEY_End,
            Gdk.KEY_Home,
            Gdk.KEY_Left,
            Gdk.KEY_Right,
            Gdk.KEY_Up,
        ]
        return self.id in keys

    def is_action_key(self) -> bool:
        """Return True if this is an action key."""

        keys = [
            Gdk.KEY_BackSpace,
            Gdk.KEY_Delete,
            Gdk.KEY_Escape,
            Gdk.KEY_KP_Enter,
            Gdk.KEY_Page_Down,
            Gdk.KEY_Page_Up,
            Gdk.KEY_Return,
            Gdk.KEY_Tab,
        ]
        return self.id in keys

    def is_alphabetic_key(self) -> bool:
        """Return True if this is an alphabetic key."""

        name = self.get_key_name()
        if len(name) != 1:
            return False

        return name.isalpha()

    def is_diacritical_key(self) -> bool:
        """Return True if this is a non-spacing diacritical key."""

        keys = [
            Gdk.KEY_dead_A,
            Gdk.KEY_dead_a,
            Gdk.KEY_dead_abovecomma,
            Gdk.KEY_dead_abovedot,
            Gdk.KEY_dead_abovereversedcomma,
            Gdk.KEY_dead_abovering,
            Gdk.KEY_dead_aboveverticalline,
            Gdk.KEY_dead_acute,
            Gdk.KEY_dead_belowbreve,
            Gdk.KEY_dead_belowcircumflex,
            Gdk.KEY_dead_belowcomma,
            Gdk.KEY_dead_belowdiaeresis,
            Gdk.KEY_dead_belowdot,
            Gdk.KEY_dead_belowmacron,
            Gdk.KEY_dead_belowring,
            Gdk.KEY_dead_belowtilde,
            Gdk.KEY_dead_belowverticalline,
            Gdk.KEY_dead_breve,
            Gdk.KEY_dead_capital_schwa,
            Gdk.KEY_dead_caron,
            Gdk.KEY_dead_cedilla,
            Gdk.KEY_dead_circumflex,
            Gdk.KEY_dead_currency,
            Gdk.KEY_dead_dasia,
            Gdk.KEY_dead_diaeresis,
            Gdk.KEY_dead_doubleacute,
            Gdk.KEY_dead_doublegrave,
            Gdk.KEY_dead_E,
            Gdk.KEY_dead_e,
            Gdk.KEY_dead_grave,
            Gdk.KEY_dead_greek,
            Gdk.KEY_dead_hook,
            Gdk.KEY_dead_horn,
            Gdk.KEY_dead_I,
            Gdk.KEY_dead_i,
            Gdk.KEY_dead_invertedbreve,
            Gdk.KEY_dead_iota,
            Gdk.KEY_dead_longsolidusoverlay,
            Gdk.KEY_dead_lowline,
            Gdk.KEY_dead_macron,
            Gdk.KEY_dead_O,
            Gdk.KEY_dead_o,
            Gdk.KEY_dead_ogonek,
            Gdk.KEY_dead_perispomeni,
            Gdk.KEY_dead_psili,
            Gdk.KEY_dead_semivoiced_sound,
            Gdk.KEY_dead_small_schwa,
            Gdk.KEY_dead_stroke,
            Gdk.KEY_dead_tilde,
            Gdk.KEY_dead_U,
            Gdk.KEY_dead_u,
            Gdk.KEY_dead_voiced_sound,
        ]

        if self.id in keys:
            return True

        name = self.get_key_name()
        if len(name) == 1:
            category = unicodedata.category(name)
            # Mn = Mark, nonspacing; Mc = Mark, spacing combining; Me = Mark, enclosing
            if category in ("Mn", "Mc", "Me"):
                return True

        return False

    def is_function_key(self) -> bool:
        """Return True if this is a function key."""

        keys = [
            Gdk.KEY_F1,
            Gdk.KEY_F2,
            Gdk.KEY_F3,
            Gdk.KEY_F4,
            Gdk.KEY_F5,
            Gdk.KEY_F6,
            Gdk.KEY_F7,
            Gdk.KEY_F8,
            Gdk.KEY_F9,
            Gdk.KEY_F10,
            Gdk.KEY_F11,
            Gdk.KEY_F12,
        ]
        return self.id in keys

    def is_locking_key(self) -> bool:
        """Return True if this is a locking key."""

        if self.is_orca_modifier():
            return self._click_count == 2

        keys = [
            Gdk.KEY_Caps_Lock,
            Gdk.KEY_Num_Lock,
            Gdk.KEY_Scroll_Lock,
            Gdk.KEY_Shift_Lock,
        ]
        return self.id in keys

    def is_modifier_key(self) -> bool:
        """Return True if this is a modifier key."""

        keys = [
            Gdk.KEY_Alt_L,
            Gdk.KEY_Alt_R,
            Gdk.KEY_Control_L,
            Gdk.KEY_Control_R,
            Gdk.KEY_Meta_L,
            Gdk.KEY_Meta_R,
            Gdk.KEY_Super_L,
            Gdk.KEY_Super_R,
            Gdk.KEY_Shift_L,
            Gdk.KEY_Shift_R,
            Gdk.KEY_ISO_Level3_Shift,
        ]
        return self.id in keys or self.is_orca_modifier()

    def is_numeric_key(self) -> bool:
        """Return True if this is a numeric key."""

        keys = [
            Gdk.KEY_0,
            Gdk.KEY_1,
            Gdk.KEY_2,
            Gdk.KEY_3,
            Gdk.KEY_4,
            Gdk.KEY_5,
            Gdk.KEY_6,
            Gdk.KEY_7,
            Gdk.KEY_8,
            Gdk.KEY_9,
            Gdk.KEY_KP_0,
            Gdk.KEY_KP_1,
            Gdk.KEY_KP_2,
            Gdk.KEY_KP_3,
            Gdk.KEY_KP_4,
            Gdk.KEY_KP_5,
            Gdk.KEY_KP_6,
            Gdk.KEY_KP_7,
            Gdk.KEY_KP_8,
            Gdk.KEY_KP_9,
        ]
        return self.id in keys

    def is_orca_modifier(self) -> bool:
        """Return True if this is the Orca modifier key."""

        if self.id == Gdk.KEY_KP_0 and self.modifiers & keybindings.SHIFT_MODIFIER_MASK:
            return orca_modifier_manager.get_manager().is_orca_modifier("KP_Insert")

        return orca_modifier_manager.get_manager().is_orca_modifier(self.keyval_name)

    def is_orca_modified(self) -> bool:
        """Return True if this key is Orca modified."""

        if self.is_orca_modifier():
            return False

        return bool(self.modifiers & keybindings.ORCA_MODIFIER_MASK)

    def _is_keypad_key(self) -> bool:
        """Return True if this is a keypad key based on keyval name or hardware keycode."""

        if self.keyval_name.startswith("KP"):
            return True
        keypad_keycodes = {63, 77, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 104, 106}
        return self.hw_code in keypad_keycodes

    def is_keypad_key_with_numlock_on(self) -> bool:
        """Return True if this is a key pad key with numlock on."""

        return self._is_kp_with_numlock

    def is_printable_key(self) -> bool:
        """Return True if this is a printable key."""

        name = self.get_key_name()
        if len(name) != 1:
            return False

        return name.isprintable()

    def is_pressed_key(self) -> bool:
        """Returns True if the key is pressed"""

        return self.type == Atspi.EventType.KEY_PRESSED_EVENT

    def is_punctuation_key(self) -> bool:
        """Return True if this is a punctuation key."""

        keys = [
            Gdk.KEY_acute,
            Gdk.KEY_ampersand,
            Gdk.KEY_apostrophe,
            Gdk.KEY_asciicircum,
            Gdk.KEY_asciitilde,
            Gdk.KEY_asterisk,
            Gdk.KEY_at,
            Gdk.KEY_backslash,
            Gdk.KEY_bar,
            Gdk.KEY_braceleft,
            Gdk.KEY_braceright,
            Gdk.KEY_bracketleft,
            Gdk.KEY_bracketright,
            Gdk.KEY_brokenbar,
            Gdk.KEY_cedilla,
            Gdk.KEY_cent,
            Gdk.KEY_colon,
            Gdk.KEY_comma,
            Gdk.KEY_copyright,
            Gdk.KEY_currency,
            Gdk.KEY_degree,
            Gdk.KEY_diaeresis,
            Gdk.KEY_dollar,
            Gdk.KEY_EuroSign,
            Gdk.KEY_equal,
            Gdk.KEY_exclam,
            Gdk.KEY_exclamdown,
            Gdk.KEY_grave,
            Gdk.KEY_greater,
            Gdk.KEY_guillemotleft,
            Gdk.KEY_guillemotright,
            Gdk.KEY_hyphen,
            Gdk.KEY_KP_Decimal,
            Gdk.KEY_KP_Add,
            Gdk.KEY_KP_Divide,
            Gdk.KEY_KP_Multiply,
            Gdk.KEY_KP_Subtract,
            Gdk.KEY_less,
            Gdk.KEY_macron,
            Gdk.KEY_minus,
            Gdk.KEY_notsign,
            Gdk.KEY_numbersign,
            Gdk.KEY_paragraph,
            Gdk.KEY_parenleft,
            Gdk.KEY_parenright,
            Gdk.KEY_percent,
            Gdk.KEY_period,
            Gdk.KEY_periodcentered,
            Gdk.KEY_plus,
            Gdk.KEY_plusminus,
            Gdk.KEY_question,
            Gdk.KEY_questiondown,
            Gdk.KEY_quotedbl,
            Gdk.KEY_quoteleft,
            Gdk.KEY_quoteright,
            Gdk.KEY_registered,
            Gdk.KEY_section,
            Gdk.KEY_semicolon,
            Gdk.KEY_slash,
            Gdk.KEY_sterling,
            Gdk.KEY_underscore,
            Gdk.KEY_yen,
        ]
        return self.id in keys

    def is_space(self) -> bool:
        """Return True if this is the space key."""

        return self.id == Gdk.KEY_space

    def get_locking_state(self) -> bool | None:
        """Returns True if the event locked a locking key, False if the event unlocked it, and None
        if not a locking key."""

        if not self.is_locking_key():
            return None

        if self.id == Gdk.KEY_Caps_Lock:
            mod = Atspi.ModifierType.SHIFTLOCK
        elif self.id == Gdk.KEY_Shift_Lock:
            mod = Atspi.ModifierType.SHIFT
        elif self.id == Gdk.KEY_Num_Lock:
            mod = Atspi.ModifierType.NUMLOCK
        else:
            return None

        return not self.modifiers & (1 << mod)

    def get_locking_state_string(self) -> str:
        """Returns the string reflecting the locking state."""

        locked = self.get_locking_state()
        if locked is None:
            return ""

        if not locked:
            return messages.LOCKING_KEY_STATE_OFF

        return messages.LOCKING_KEY_STATE_ON

    def get_key_name(self) -> str:
        """Returns the string to be used for presenting the key."""

        if self._text.strip() and self._text.isprintable():
            return self._text

        name = keynames.get_key_name(self.keyval_name)
        if name is not None:
            return name

        unicode_codepoint = Gdk.keyval_to_unicode(self.id)
        if unicode_codepoint:
            char = chr(unicode_codepoint)
            if char.isprintable():
                return char

        return self.keyval_name

    def get_object(self) -> Atspi.Accessible | None:
        """Returns the object believed to be associated with this key event."""

        return self._obj

    def set_object(self, obj: Atspi.Accessible | None) -> None:
        """Sets the object believed to be associated with this key event."""

        module_name = inspect.getmodulename(inspect.stack()[1].filename)
        if not (module_name and module_name.startswith("input_event")):
            raise PermissionError("Unauthorized setter of input event property")

        self._obj = obj

    def get_window(self) -> Atspi.Accessible | None:
        """Returns the window believed to be associated with this key event."""

        return self._window

    def set_window(self, window: Atspi.Accessible | None) -> None:
        """Sets the window believed to be associated with this key event."""

        module_name = inspect.getmodulename(inspect.stack()[1].filename)
        if not (module_name and module_name.startswith("input_event")):
            raise PermissionError("Unauthorized setter of input event property")

        self._window = window

    def get_script(self) -> default.Script | None:
        """Returns the script believed to be associated with this key event."""

        return self._script

    def set_script(self, script: default.Script | None) -> None:
        """Sets the script believed to be associated with this key event."""

        module_name = inspect.getmodulename(inspect.stack()[1].filename)
        if not (module_name and module_name.startswith("input_event")):
            raise PermissionError("Unauthorized setter of input event property")

        self._script = script

    def _present(self) -> None:
        if not self._script:
            return

        if self.is_pressed_key():
            presentation_manager.get_manager().interrupt_presentation()

        # pylint: disable=import-outside-toplevel
        from . import learn_mode_presenter, sleep_mode_manager

        if learn_mode_presenter.get_presenter().is_active():
            return

        if sleep_mode_manager.get_manager().is_active_for_app(self._script.app):
            return

        presentation_manager.get_manager().present_keyboard_event(self)

    def process(self) -> None:
        """Processes this input event."""

        start_time = time.time()
        should_obscure = self.should_obscure()
        if not should_obscure:
            data = f"'{self.keyval_name}' ({self.hw_code})"
        else:
            data = "(obscured)"

        debug.print_message(debug.LEVEL_INFO, f"\n{self}")

        msg = f"\nvvvvv PROCESS {self.type.value_name.upper()}: {data} vvvvv"
        debug.print_message(debug.LEVEL_INFO, msg, False)

        tokens = ["SCRIPT:", self._script]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)

        tokens = ["WINDOW:", self._window]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)

        tokens = ["LOCATION:", self._obj]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)

        if self._script:
            script = self._script
            command = command_manager.get_manager().get_command_for_event(self)
            if not should_obscure and command is not None:
                tokens = ["COMMAND:", command.get_name()]
                debug.print_tokens(debug.LEVEL_INFO, tokens, True)

            # pylint: disable-next=import-outside-toplevel
            from . import learn_mode_presenter

            learn_mode = learn_mode_presenter.get_presenter().is_active()
            if learn_mode:
                tokens = ["KEYBOARD EVENT: Learn mode is active"]
                debug.print_tokens(debug.LEVEL_INFO, tokens, True)
                self._handler = lambda: learn_mode_presenter.get_presenter().handle_event(
                    self,
                    command,
                )
            elif command is not None and command.is_enabled():
                self._handler = lambda: command.execute(script, self)

        if self.is_orca_modifier():
            if self._click_count == 2:
                orca_modifier_manager.get_manager().toggle_modifier(self)

        if self.is_pressed_key() and self.keyval_name == "Num_Lock":
            numlock_on = self.get_locking_state()
            if numlock_on is not None:
                command_manager.get_manager().handle_numlock_toggled(numlock_on)

        self._present()

        if self.is_pressed_key() and self._handler:
            GLib.timeout_add(1, self._handle)

        msg = f"TOTAL PROCESSING TIME: {time.time() - start_time:.4f}"
        debug.print_message(debug.LEVEL_INFO, msg, True)

        msg = f"^^^^^ PROCESS {self.type.value_name.upper()}: {data} ^^^^^\n"
        debug.print_message(debug.LEVEL_INFO, msg, False)

    def _handle(self) -> bool:
        """Handles this event after a timeout. Returns False to stop the timeout."""

        start_time = time.time()
        data = f"'{self.keyval_name}' ({self.hw_code})"
        msg = f"\nvvvvv HANDLE {self.type.value_name.upper()}: {data} vvvvv"
        debug.print_message(debug.LEVEL_INFO, msg, False)

        if self._handler:
            try:
                self._handler()
            except GLib.GError as error:
                msg = f"KEYBOARD EVENT: Exception calling handler: {error}"
                debug.print_message(debug.LEVEL_WARNING, msg, True)

        msg = f"TOTAL PROCESSING TIME: {time.time() - start_time:.4f}"
        debug.print_message(debug.LEVEL_INFO, msg, True)

        msg = f"^^^^^ HANDLE {self.type.value_name.upper()}: {data} ^^^^^\n"
        debug.print_message(debug.LEVEL_INFO, msg, False)

        return False


class BrailleEvent(InputEvent):
    """Provides support for handling braille events."""

    def __init__(self, event: dict) -> None:
        super().__init__(BRAILLE_EVENT)
        self.event: dict = event
        self._script: default.Script | None = script_manager.get_manager().get_active_script()

    def __str__(self) -> str:
        return f"{self.type.upper()} {self.event}"

    def get_command(self) -> command_manager.BrailleCommand | None:
        """Returns the BrailleCommand associated with this event."""

        braille_key: int = self.event["command"]
        command = command_manager.get_manager().get_command_for_braille_event(braille_key)
        tokens = [f"BRAILLE EVENT: Command for braille key {braille_key} is", command]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
        return command

    def process(self):
        """Processes this event."""

        tokens = ["\nvvvvv PROCESS", self, "vvvvv"]
        debug.print_tokens(debug.LEVEL_INFO, tokens, False)

        start_time = time.time()
        result = self._process()
        msg = f"TOTAL PROCESSING TIME: {time.time() - start_time:.4f}"
        debug.print_message(debug.LEVEL_INFO, msg, False)

        tokens = ["^^^^^ PROCESS", self, "^^^^^"]
        debug.print_tokens(debug.LEVEL_INFO, tokens, False)
        return result

    def _process(self):
        # pylint: disable-next=import-outside-toplevel
        from . import learn_mode_presenter

        presenter = learn_mode_presenter.get_presenter()

        command = self.get_command()
        if not command:
            if presenter.is_active():
                tokens = ["BRAILLE EVENT: No command, but in learn mode", self]
                debug.print_tokens(debug.LEVEL_INFO, tokens, True)
                return True

            tokens = ["BRAILLE EVENT: No command found for", self]
            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
            return False

        tokens = ["BRAILLE EVENT: Command is:", command]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)

        if presenter.is_active():
            if presenter.handle_braille_event(self._script, self, command):
                return True

        command.execute(self._script, self)
        return True


class MouseButtonEvent(InputEvent):
    """Provides support for handling mouse button events."""

    # TODO - JD: Remove this and the validation logic once we have a fix for
    # https://gitlab.gnome.org/GNOME/at-spi2-core/-/issues/194.
    try:
        display = Gdk.Display.get_default()  # pylint: disable=no-value-for-parameter
        seat = Gdk.Display.get_default_seat(display)
        _pointer = seat.get_pointer()
    except Exception:  # pylint: disable=broad-exception-caught
        _pointer = None

    def __init__(self, event):
        super().__init__(MOUSE_BUTTON_EVENT)
        self.x = event.detail1
        self.y = event.detail2
        self.pressed = event.type.endswith("p")
        self.button = event.type[len("mouse:button:") : -1]
        self._script = script_manager.get_manager().get_active_script()
        self.window = focus_manager.get_manager().get_active_window()
        self.app = None

        if self.pressed:
            self._validate_coordinates()

        if not self._script:
            return

        if not AXUtilities.can_be_active_window(self.window):
            self.window = AXUtilities.find_active_window()

        if not self.window:
            return

        self.app = AXUtilities.get_application(self.window)

    def _validate_coordinates(self):
        if not self._pointer:
            return

        x, y = self._pointer.get_position()[1:]
        if math.sqrt((self.x - x) ** 2 + (self.y - y) ** 2) < 25:
            return

        msg = (
            f"WARNING: Event coordinates ({self.x}, {self.y}) may be bogus. Updating to ({x}, {y})"
        )
        debug.print_message(debug.LEVEL_INFO, msg, True)
        self.x, self.y = x, y


class RemoteControllerEvent(InputEvent):
    """A simple input event whose main purpose is identification of the origin."""

    def __init__(self):
        super().__init__(REMOTE_CONTROLLER_EVENT)



# Orca
#
# Copyright 2006-2008 Sun Microsystems Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=too-many-branches

"""Manager for user's pronunciation dictionary that maps words to what they sound like."""

from __future__ import annotations

from typing import TYPE_CHECKING

import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk  # pylint: disable=no-name-in-module

from . import (
    gsettings_migrator,
    gsettings_registry,
    guilabels,
    messages,
    preferences_grid_base,
    presentation_manager,
    script_manager,
)

if TYPE_CHECKING:
    from collections.abc import Callable

    from .scripts import default


class PronunciationDictionaryPreferencesGrid(  # pylint: disable=too-many-instance-attributes
    preferences_grid_base.PreferencesGridBase,
):
    """GtkGrid containing the Pronunciation Dictionary preferences page."""

    # pylint: disable=no-member

    def __init__(self, manager: PronunciationDictionaryManager, script: default.Script) -> None:
        super().__init__(guilabels.PRONUNCIATION)
        self._manager: PronunciationDictionaryManager = manager
        self._script: default.Script = script
        self._initializing: bool = True

        self._pronunciations: list[tuple[str, str]] = []
        self._inherited_keys: set[str] = set()
        self._deleted_inherited_keys: set[str] = set()
        self._listbox: Gtk.ListBox | None = None
        self._has_unsaved_changes: bool = False
        self._loaded_from_settings: bool = False

        # Size group to ensure all left labels (phrases) have the same width
        self._left_label_size_group: Gtk.SizeGroup = Gtk.SizeGroup(
            mode=Gtk.SizeGroupMode.HORIZONTAL,
        )

        self._build()
        self.refresh()

    def _build(self) -> None:
        """Create the Gtk widgets composing the grid."""

        row = 0

        # Info box
        info_listbox = self._create_info_listbox(guilabels.PRONUNCIATION_DICTIONARY_INFO)
        info_listbox.set_margin_bottom(12)
        self.attach(info_listbox, 0, row, 1, 1)
        row += 1

        # Header box with title and + button
        header_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=12)
        header_box.set_margin_bottom(6)

        title_label = Gtk.Label(label=guilabels.PRONUNCIATION_DICTIONARY)
        title_label.set_halign(Gtk.Align.START)
        title_label.get_style_context().add_class("heading")
        header_box.pack_start(title_label, True, True, 0)

        add_button = Gtk.Button.new_from_icon_name("list-add-symbolic", Gtk.IconSize.BUTTON)
        add_button.get_accessible().set_name(guilabels.DICTIONARY_NEW_ENTRY)
        add_button.connect("clicked", self._on_add_clicked)
        header_box.pack_end(add_button, False, False, 0)

        self.attach(header_box, 0, row, 1, 1)
        row += 1

        # ListBox for pronunciation entries
        self._listbox = Gtk.ListBox()
        self._listbox.set_selection_mode(Gtk.SelectionMode.SINGLE)
        self._listbox.get_style_context().add_class("frame")

        # Suppress pronunciation substitution while editing entries
        self._listbox.connect("realize", self._on_listbox_realize)
        self._listbox.connect("unrealize", self._on_listbox_unrealize)

        scrolled_window = self._create_scrolled_window(self._listbox)
        self.attach(scrolled_window, 0, row, 1, 1)

    def _create_two_label_row(  # pylint: disable=too-many-arguments,too-many-positional-arguments,too-many-locals
        self,
        left_label_text: str,
        right_label_text: str,
        edit_handler: Callable[[Gtk.Button], None] | None = None,
        delete_handler: Callable[[Gtk.Button], None] | None = None,
        include_top_separator: bool = True,
        left_label_size_group: Gtk.SizeGroup | None = None,
    ) -> Gtk.ListBoxRow:
        """Create a single listbox row with two labels and optional edit/delete buttons."""

        row = Gtk.ListBoxRow()
        row.set_activatable(False)

        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)

        if include_top_separator:
            separator = Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL)
            vbox.pack_start(separator, False, False, 0)

        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=12)
        hbox.set_margin_start(12)
        hbox.set_margin_end(12)
        hbox.set_margin_top(12)
        hbox.set_margin_bottom(12)

        left_label = Gtk.Label(label=left_label_text, xalign=0)

        if left_label_size_group:
            left_label_size_group.add_widget(left_label)

        hbox.pack_start(left_label, False, False, 0)

        right_label = Gtk.Label(label=right_label_text, xalign=0)
        right_label.set_hexpand(True)
        hbox.pack_start(right_label, True, True, 0)

        button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)

        delete_button = None
        if delete_handler:
            delete_button = Gtk.Button.new_from_icon_name("user-trash-symbolic", Gtk.IconSize.DND)
            delete_button.set_relief(Gtk.ReliefStyle.NONE)
            delete_button.get_accessible().set_name(guilabels.DICTIONARY_DELETE)
            delete_button.connect("clicked", delete_handler)
            button_box.pack_start(delete_button, False, False, 0)

        edit_button = None
        if edit_handler:
            edit_button = Gtk.Button.new_from_icon_name("document-edit-symbolic", Gtk.IconSize.DND)
            edit_button.set_relief(Gtk.ReliefStyle.NONE)
            edit_button.get_accessible().set_name(guilabels.DIALOG_EDIT)
            edit_button.connect("clicked", edit_handler)
            button_box.pack_start(edit_button, False, False, 0)

        hbox.pack_end(button_box, False, False, 0)

        vbox.pack_start(hbox, False, False, 0)
        row.add(vbox)

        row.delete_button = delete_button
        row.edit_button = edit_button

        return row

    def _create_row(
        self,
        phrase: str,
        substitution: str,
        row_index: int,
        include_top_separator: bool = True,
    ) -> Gtk.ListBoxRow:
        """Create a ListBoxRow for a pronunciation entry."""

        row = self._create_two_label_row(
            phrase,
            substitution,
            edit_handler=self._on_edit_clicked,
            delete_handler=self._on_delete_clicked,
            include_top_separator=include_top_separator,
            left_label_size_group=self._left_label_size_group,
        )

        row.pronunciation_row_index = row_index

        if row.edit_button:
            row.edit_button.pronunciation_row_index = row_index

        if row.delete_button:
            row.delete_button.pronunciation_row_index = row_index

        return row

    def _on_edit_clicked(self, button: Gtk.Button) -> None:
        """Handle edit button click."""

        row_index = button.pronunciation_row_index
        phrase, substitution = self._pronunciations[row_index]
        self._show_edit_dialog(phrase, substitution, row_index)

    def _on_delete_clicked(self, button: Gtk.Button) -> None:
        """Handle delete button click."""

        row_index = button.pronunciation_row_index
        phrase, _ = self._pronunciations[row_index]
        key = phrase.lower()
        if key in self._inherited_keys:
            self._deleted_inherited_keys.add(key)
            self._inherited_keys.discard(key)
        del self._pronunciations[row_index]
        self._has_unsaved_changes = True
        self.refresh()

        script = script_manager.get_manager().get_active_script()
        if script:
            presentation_manager.get_manager().present_message(
                messages.PRONUNCIATION_DELETED % phrase,
            )

    def _on_listbox_realize(self, _widget: Gtk.Widget) -> None:
        """Suppress pronunciation substitution while editing entries."""

        self._manager.suppress()

    def _on_listbox_unrealize(self, _widget: Gtk.Widget) -> None:
        """Restore pronunciation substitution when done editing entries."""

        self._manager.unsuppress()

    def _on_add_clicked(self, _button: Gtk.Button) -> None:
        """Handle Add button click to open add dialog."""

        self._show_add_dialog()

    def _show_add_dialog(self) -> None:
        """Show dialog to add a new pronunciation."""

        dialog, add_button = self._create_header_bar_dialog(
            guilabels.ADD_NEW_PRONUNCIATION,
            guilabels.DIALOG_CANCEL,
            guilabels.DIALOG_ADD,
        )

        content_area = dialog.get_content_area()

        listbox = preferences_grid_base.FocusManagedListBox()

        # Size group to ensure labels have same width and entries align
        label_size_group = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)

        # Actual string row
        phrase_entry = Gtk.Entry()
        phrase_entry.set_size_request(-1, 40)
        phrase_row = self._create_labeled_entry_row(
            guilabels.DICTIONARY_ACTUAL_STRING,
            phrase_entry,
            include_top_separator=False,
            label_size_group=label_size_group,
        )
        listbox.add_row_with_widget(phrase_row, phrase_entry)

        # Replacement string row
        substitution_entry = Gtk.Entry()
        substitution_entry.set_size_request(-1, 40)
        substitution_row = self._create_labeled_entry_row(
            guilabels.DICTIONARY_REPLACEMENT_STRING,
            substitution_entry,
            label_size_group=label_size_group,
        )
        listbox.add_row_with_widget(substitution_row, substitution_entry)

        def on_entry_activate(_entry):
            """Only activate dialog if both fields are filled."""
            if phrase_entry.get_text().strip() and substitution_entry.get_text().strip():
                dialog.response(Gtk.ResponseType.OK)

        phrase_entry.connect("activate", on_entry_activate)
        substitution_entry.connect("activate", on_entry_activate)

        content_area.pack_start(listbox, True, True, 0)

        def on_response(dlg, response_id):
            if response_id == Gtk.ResponseType.OK:
                phrase = phrase_entry.get_text().strip()
                substitution = substitution_entry.get_text().strip()
                if phrase and substitution:
                    self._pronunciations.append((phrase, substitution))
                    self._has_unsaved_changes = True
                    self.refresh()
            dlg.destroy()

        dialog.connect("response", on_response)
        dialog.show_all()
        add_button.grab_default()
        phrase_entry.grab_focus()

    def _show_edit_dialog(  # pylint: disable=too-many-locals
        self,
        phrase: str,
        substitution: str,
        row_index: int,
    ) -> None:
        """Show dialog to edit an existing pronunciation."""

        dialog, edit_button = self._create_header_bar_dialog(
            guilabels.EDIT_PRONUNCIATION,
            guilabels.DIALOG_CANCEL,
            guilabels.DIALOG_EDIT,
        )

        content_area = dialog.get_content_area()

        listbox = preferences_grid_base.FocusManagedListBox()

        # Size group to ensure labels have same width and entries align
        label_size_group = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)

        # Actual string row
        phrase_entry = Gtk.Entry()
        phrase_entry.set_text(phrase)
        phrase_entry.set_size_request(-1, 40)
        phrase_row = self._create_labeled_entry_row(
            guilabels.DICTIONARY_ACTUAL_STRING,
            phrase_entry,
            include_top_separator=False,
            label_size_group=label_size_group,
        )
        listbox.add_row_with_widget(phrase_row, phrase_entry)

        # Replacement string row
        substitution_entry = Gtk.Entry()
        substitution_entry.set_text(substitution)
        substitution_entry.set_size_request(-1, 40)
        substitution_row = self._create_labeled_entry_row(
            guilabels.DICTIONARY_REPLACEMENT_STRING,
            substitution_entry,
            label_size_group=label_size_group,
        )
        listbox.add_row_with_widget(substitution_row, substitution_entry)

        def on_entry_activate(_entry):
            """Only activate dialog if both fields are filled."""
            if phrase_entry.get_text().strip() and substitution_entry.get_text().strip():
                dialog.response(Gtk.ResponseType.OK)

        phrase_entry.connect("activate", on_entry_activate)
        substitution_entry.connect("activate", on_entry_activate)

        content_area.pack_start(listbox, True, True, 0)

        def on_response(dlg, response_id):
            if response_id == Gtk.ResponseType.OK:
                new_phrase = phrase_entry.get_text().strip()
                new_substitution = substitution_entry.get_text().strip()
                if new_phrase and new_substitution:
                    self._inherited_keys.discard(phrase.lower())
                    self._pronunciations[row_index] = (new_phrase, new_substitution)
                    self._has_unsaved_changes = True
                    self.refresh()
            dlg.destroy()

        dialog.connect("response", on_response)
        dialog.show_all()
        edit_button.grab_default()
        phrase_entry.grab_focus()

    def reload(self) -> None:
        """Reload settings from the manager and refresh the UI."""

        self._pronunciations = []
        self._inherited_keys = set()
        self._deleted_inherited_keys = set()
        self._loaded_from_settings = False
        self._has_unsaved_changes = False
        self.refresh()

    def save_settings(self, profile: str = "", app_name: str = "") -> dict[str, list[str]]:
        """Save settings and return a dictionary of the current values for those settings."""

        self._manager.set_dictionary({})
        result: dict[str, list[str]] = {}

        for phrase, substitution in self._pronunciations:
            if phrase and substitution:
                self._manager.set_pronunciation(phrase, substitution)
                result[phrase.lower()] = [phrase, substitution]

        self._has_unsaved_changes = False

        if profile:
            registry = gsettings_registry.get_registry()
            parent_pronunciations: dict[str, str] = {}
            if profile != "default" or app_name:
                if profile != "default":
                    parent_pronunciations |= registry.get_pronunciations("default", "")
                if app_name:
                    parent_pronunciations |= registry.get_pronunciations(profile, "")

            diff: dict[str, str] = {}
            for key, value in result.items():
                replacement = value[1]
                if parent_pronunciations.get(key) != replacement:
                    diff[key] = replacement
            for key in self._deleted_inherited_keys:
                if key in parent_pronunciations:
                    diff[key] = ""

            pron_gs = registry.get_settings("pronunciations", profile, "pronunciations", app_name)
            if pron_gs is not None:
                if diff:
                    gsettings_migrator.import_pronunciations(pron_gs, diff)
                elif pron_gs.get_user_value("entries") is not None:
                    pron_gs.reset("entries")

        return result

    def refresh(self) -> None:
        """Update listbox to reflect current pronunciation list."""

        if self._listbox is None:
            return

        self._initializing = True

        for child in self._listbox.get_children():
            self._listbox.remove(child)

        if not self._loaded_from_settings:
            self._loaded_from_settings = True
            registry = gsettings_registry.get_registry()
            profile = registry.get_active_profile()
            app_name = ""
            if self._script and self._script.app:
                from .ax_object import AXObject  # pylint: disable=import-outside-toplevel

                app_name = AXObject.get_name(self._script.app)

            pronunciation_dict = registry.layered_lookup(
                "pronunciations",
                "entries",
                "a{ss}",
                app_name=app_name or None,
                default={},
            )
            local_dict = registry.get_pronunciations(profile, app_name)
            self._inherited_keys = set(pronunciation_dict.keys()) - set(local_dict.keys())

            for key in sorted(pronunciation_dict.keys()):
                if pronunciation_dict[key]:
                    self._pronunciations.append((key, pronunciation_dict[key]))

        if self._pronunciations:
            for index, (phrase, substitution) in enumerate(self._pronunciations):
                row = self._create_row(
                    phrase,
                    substitution,
                    index,
                    include_top_separator=index > 0,
                )
                self._listbox.add(row)
        else:
            empty_row = self._create_info_row(guilabels.DICTIONARY_EMPTY)
            self._listbox.add(empty_row)

        self._listbox.show_all()
        self._initializing = False


@gsettings_registry.get_registry().gsettings_schema(
    "org.gnome.Orca.Pronunciations",
    name="pronunciations",
)
class PronunciationDictionaryManager:
    """Manager for the pronunciation dictionary."""

    def __init__(self) -> None:
        self._dictionary: dict[str, str] | None = None
        self._cached_app: str | None = None
        self._cached_profile: str = "default"
        self._suppressed: bool = False

    def create_preferences_grid(
        self,
        script: default.Script,
    ) -> PronunciationDictionaryPreferencesGrid:
        """Returns the GtkGrid containing the pronunciation dictionary UI."""

        return PronunciationDictionaryPreferencesGrid(self, script)

    def suppress(self) -> None:
        """Suppresses pronunciation substitution without changing the user's preference."""

        self._suppressed = True

    def unsuppress(self) -> None:
        """Restores pronunciation substitution after a prior suppress call."""

        self._suppressed = False

    def get_pronunciation(self, word: str) -> str:
        """Returns the pronunciation for word, or word if not found."""

        if self._suppressed:
            return word

        registry = gsettings_registry.get_registry()
        current_app = registry.get_active_app()
        current_profile = registry.get_active_profile()
        if self._cached_app != current_app or self._cached_profile != current_profile:
            self._dictionary = None
            self._cached_app = current_app
            self._cached_profile = current_profile

        if self._dictionary is None:
            self._dictionary = registry.layered_lookup(
                "pronunciations",
                "entries",
                "a{ss}",
                default={},
            )

        return self._dictionary.get(word.lower(), word) or word

    def set_pronunciation(self, word: str, replacement: str) -> None:
        """Adds word/replacement pair."""

        # TODO - JD: Storing the words as lowercase is what we've done historically.
        # However, this means that on occasions where case sensitivity matters, there
        # will be a false positive (e.g., "US" vs "us"). Consider adding a checkbox
        # to the UI to allow users to choose case sensitivity for individual entries.

        key = word.lower()
        if self._dictionary is None:
            self._dictionary = {}
        self._dictionary[key] = replacement

    @gsettings_registry.get_registry().gsetting(
        key="entries",
        schema="pronunciations",
        gtype="a{ss}",
        default={},
        summary="Pronunciation dictionary entries",
    )
    def get_dictionary(self) -> dict[str, str]:
        """Returns the pronunciation dictionary."""

        if self._dictionary is None:
            return {}
        return self._dictionary

    def set_dictionary(self, value: dict[str, str]) -> None:
        """Sets the pronunciation dictionary, or invalidates the cache if empty."""

        self._dictionary = value or None


_manager = PronunciationDictionaryManager()


def get_manager() -> PronunciationDictionaryManager:
    """Returns the pronunciation-dictionary-manager singleton."""

    return _manager
